package com.unissoft.basic.controller;


import java.util.Date;
import java.util.List;

import com.unissoft.common.*;
import com.unissoft.mqProducer.Producer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.basic.model.BasicEnum;
import com.unissoft.basic.service.BasicEnumService;
import com.unissoft.parameter.model.HardwareParameter;

import io.swagger.annotations.ApiOperation;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@RestController
@RequestMapping("/basic-enum")
public class BasicEnumController {
    @Autowired
    private Producer producer;

	private BasicEnumService enumService;

    @Autowired
    public void setBasicEnumController(BasicEnumService enumService) {
        this.enumService = enumService;
    }

    /**
     * 分页查询列表
     * @param pageParam
     * @return
     */
    @PostMapping("/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        IPage<BasicEnum> page = enumService.getPage(pageParam);
        return ResultView.success(page);
    }
    

    /**
     * 新增
     * @param basicEnum
     * @return
     */
    @SystemLog(operModul = "参数设置", operType = "数据字典", operDesc = "数据字典新增")
    @PostMapping("/save")
    public ResultView save(@RequestBody BasicEnum basicEnum) {
    	Date date = new Date();
    	basicEnum.setCreateTime(date);
    	basicEnum.setUpdateTime(date);
        boolean save = enumService.save(basicEnum);
        if (save) {
            producer.send(basicEnum, MessageEnum.SAVE, ServiceEnum.BASIC_ENUM);
            return ResultView.success();
        }else {
            return ResultView.error();
        }
    }
    
    
    /**
     * 修改
     * @param basicEnum
     * @return
     */
    @SystemLog(operModul = "参数设置", operType = "数据字典", operDesc = "数据字典修改")
    @PostMapping("/update")
    public ResultView update(@RequestBody BasicEnum basicEnum) {
    	basicEnum.setUpdateTime(new Date());
        boolean b = enumService.updateById(basicEnum);
        if (b) {
            //发送消息
            producer.send(basicEnum, MessageEnum.UPDATE,ServiceEnum.BASIC_ENUM);
            return ResultView.success();
        }else {
            return ResultView.error();
        }
    }
    
    
    /**
	 * 获取数据字典json树结构
	 * @return
	 * @throws Exception 
	 */
    @SystemLog(operModul = "参数设置", operType = "数据字典", operDesc = "数据字典查询")
    @PostMapping("/getTree")
	@ApiOperation(value="查询字典数据树", notes="查询字典数据树")
	public ResultView getTree(@RequestBody PageParam pageParam) {
		List<BasicEnum> list = enumService.queryByTree(pageParam);
		return ResultView.success(list);
	}
}
