package com.unissoft.interaction.algorithm;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * 圆筒仓
 * @author My Sunshine
 *
 */
public class CylinderAlg {

	/**
	 * 定义常量  减速机传动比
	 */
	public static int basicData = 50;

    /**
     * 打点要发送的角度  
     *
     * @param diameter
     * @param high
     * @return
     */
    public static String getAveragePosition(double diameter, double high) {
    	StringBuffer sb = new StringBuffer();

        double x1 = 0;
        double y1 = 0;
        //打出来三角形的底边AB 实际是平面上点的斜边
        double AB = 0; //理想状态
        double x = 0;
        double y = 0;
        double BC = 0;
        double vertical_angle = 0;//垂直的角度
        double level_angle = 0;//水平的角度
       
        //根据直径判断半径是整数还是非整数 如果是非整数 则向下取整
        double r = Math.floor(diameter / 2);  //最大的半径 即一共有多少个圆圈

        //一共有多少个圆圈
        int circleNumber = (int) r;

        for (int i = 1; i <= circleNumber; i++) {
            //每一个圆一共有多少个的点   此时的半径长就是i
            int pointCount = (int) ((2 * Math.PI * i) / 1.57);

            for (int j = 0; j < pointCount; j++) {

                /**
                 * 以其他点为圆心的圆  r是i degrees是以圆心为中心转动的角度 其坐标x1、y1则是
                 */
                double degrees = (360 / pointCount) * j;
                //System.out.println(degrees+"度数");
                double radians = Math.toRadians(degrees);//转化为弧度
                String strx = String.format("%.2f%n", Math.cos(radians) * i);//计算横坐标的值
                x1 = new Double(strx);
                String stry = String.format("%.2f%n", Math.sin(radians) * i);//计算纵坐标的值
                y1 = new Double(stry);

                /**
                 * 以设备为中心，把圆上的点分为上下两半，
                 * 新的圆 是旧的圆向左平移了(diameter/2-i)+i 个长度 得到的
                 * (设备在新的圆的最左边上，设备是坐标系的原点)
                 * 所以 横坐标加上这个长度  纵坐标不变
                 */

                //纵坐标 无论是处于上半部分还是下半部分 其纵坐标是不变的
                x = x1 + (diameter / 2 - i) + i;
                y = y1;
                //System.out.print("["+x+","+y+"]"+",");
                //底面三角形的斜边
                AB = Math.sqrt(x * x + y * y);
                //垂直三角形的斜边
                BC = Math.sqrt(AB * AB + high * high);
                if (x == 0 && y == 0) {//圆 点
                    level_angle = 0;
                    vertical_angle = 0;

                } else if (y < 0) {//反向转
                    level_angle = -Math.toDegrees(Math.acos(Math.abs(y) / AB));
                    vertical_angle = -(Math.toDegrees(Math.acos(high / BC)));

                } else {//正向转
                    level_angle = Math.toDegrees(Math.acos(y / AB));
                    vertical_angle = Math.toDegrees(Math.acos(high / BC));

                }

                sb.append(Math.round(vertical_angle) * basicData);
                sb.append(",");
                sb.append(Math.round(level_angle) * basicData);
                sb.append(";");

                // sb.append("\n");
            }
        }

        /**
         * 追加一个圆筒仓中心的点，其坐标是(diameter/2,0) 水平转动90度 步长为2167
         */
        AB = diameter / 2;//垂直三角形的底边
        BC = Math.sqrt(AB * AB + high * high);//垂直三角形的斜边
        vertical_angle = Math.toDegrees(Math.acos(high / BC));
        sb.append(Math.round(vertical_angle) * basicData);
        sb.append(",");
        sb.append(90 * basicData);
        //System.out.println(sb);
        
        return sb.toString();

    }
    
    
    
    /**
     * 打点要发送的角度  
     *
     * @param diameter
     * @param high
     * @return
     */
    public static String getAveragePosition1(double diameter, double high) {

    	StringBuffer sb = new StringBuffer();

        double x1 = 0;
        double y1 = 0;
        //打出来三角形的底边AB 实际是平面上点的斜边
        double AB = 0; //理想状态
        double x = 0;
        double y = 0;
        double BC = 0;
        double vertical_angle = 0;//垂直的角度
        double level_angle = 0;//水平的角度
       
        //根据直径判断半径是整数还是非整数 如果是非整数 则向下取整
        double r = Math.floor(diameter / 2);  //最大的半径 即一共有多少个圆圈

        //一共有多少个圆圈
        int circleNumber = (int) r;

        for (int i = 1; i <= circleNumber; i++) {
            //每一个圆一共有多少个的点   此时的半径长就是i
            int pointCount = (int) ((2 * Math.PI * i) / 1.57);

            for (int j = 0; j < pointCount; j++) {

                /**
                 * 以其他点为圆心的圆  r是i degrees是以圆心为中心转动的角度 其坐标x1、y1则是
                 */
                double degrees = (360 / pointCount) * j;
                //System.out.println(degrees+"度数");
                double radians = Math.toRadians(degrees);//转化为弧度
                String strx = String.format("%.2f%n", Math.cos(radians) * i);//计算横坐标的值
                x1 = new Double(strx);
                String stry = String.format("%.2f%n", Math.sin(radians) * i);//计算纵坐标的值
                y1 = new Double(stry);

                /**
                 * 以设备为中心，把圆上的点分为上下两半，
                 * 新的圆 是旧的圆向左平移了(diameter/2-i)+i 个长度 得到的
                 * (设备在新的圆的最左边上，设备是坐标系的原点)
                 * 所以 横坐标加上这个长度  纵坐标不变
                 */

                //纵坐标 无论是处于上半部分还是下半部分 其纵坐标是不变的
                x = x1 + (diameter / 2 - i) + i;
                y = y1;
                //System.out.print("["+x+","+y+"]"+",");
                //底面三角形的斜边
                AB = Math.sqrt(x * x + y * y);
                //垂直三角形的斜边
                BC = Math.sqrt(AB * AB + high * high);
                
                if (x == 0 && y == 0) {//圆 点
                    level_angle = 0;
                    vertical_angle = 0;

                } else if (y < 0) {//正向转
                    level_angle = Math.toDegrees(Math.asin(Math.abs(y) / AB));
                    vertical_angle = (Math.toDegrees(Math.acos(high / BC)));

                } else if(y == 0 && x != 0){//直径所在的直线
                	level_angle = 0;
                	vertical_angle = Math.toDegrees(Math.acos(high / BC));
                }else {//正向转
                	level_angle = -Math.toDegrees(Math.asin(y / AB));
                    vertical_angle = Math.toDegrees(Math.acos(high / BC));
                }

                
                sb.append(Math.round(vertical_angle) * basicData);
                sb.append(",");
                sb.append(Math.round(level_angle) * basicData);
                sb.append(";");
                //System.out.println("["+x+","+y+"]"+","+"************"+vertical_angle+","+level_angle);
                //System.out.println(sb);
                // sb.append("\n");
            }
        }

        /**
         * 追加一个圆筒仓中心的点，其坐标是(diameter/2,0) 水平转动0 
         */
        AB = diameter / 2;//垂直三角形的底边
        BC = Math.sqrt(AB * AB + high * high);//垂直三角形的斜边
        vertical_angle = Math.toDegrees(Math.acos(high / BC));
        sb.append(Math.round(vertical_angle) * basicData);
        sb.append(",");
        sb.append(0);
        //System.out.println(sb);
        
        return sb.toString();

    }


    /**
     * 计算  以粮面为基准面的高
     *
     * @param rotationxy
     * @return
     */
    /*public static double zHeight(String rotationxy, double diameter, double high) {

        List<Double> zList = new ArrayList<Double>();

        // logger.info("zHeight - list",list);
        double AC = high;
        double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面到设备的水平距离
        double z = 0; //实际点的坐标的z轴
        double zHigh = 0;//以粮面为基准面的高
        double zMax = 0;//最高的点
        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离
        BigDecimal baseNumber = new BigDecimal(basicData);

        //  "18,7,17.88 - 19,6,28.88 - 17,4,39.88"  垂直角度，水平角度，返回距离 -
        String[] split = rotationxy.split(";");
        for (int i = 0; i < split.length; i++) {
            //System.out.println("split[i]是:"+split[i]);
            String[] split1 = split[i].split(",");
            verticalAngle = new BigDecimal(split1[0]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            levelAngle = new BigDecimal(split1[1]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            realHypotenuse = new BigDecimal(split1[2]).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).doubleValue(); //单位转化为米

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinc = new Double(verticalAngleS);
            //B1C = realHypotenuse;
            A1B1 = realHypotenuse * sinc; //即 平面三角形的斜边

            //logger.info("zHeight - A1B1 {} ",A1B1);

            String sinLevel = String.format("%.2f%n", Math.sin(Math.abs(levelAngle)));
            double sinLevelc = new Double(sinLevel);

            
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             * 
            //打到墙上 最大的边
            double maxLength = sinLevelc * diameter;

            if (A1B1 < maxLength) {
                //实际打在粮面上的点的坐标
                String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
                double cosc = new Double(verticalAngleC);
                double A1C = realHypotenuse * cosc;
                zList.add(A1C);
                // logger.info("zHeight - z√√√√√√z z z z z {} ",z);

            }
        }
        zHigh = Collections.min(zList);
        return zHigh;
	
    }*/


    /**
     * 计算体积
     *
     * @param zList 粮面高集合(z值的集合)
     */
    public static double calculateVolume(List<Double> zList, double diameter, double h1, double h2) {

        //17.1  18.3  19.2  18.3    20.1  16.4  19.8  16.7  19.0  18.9
        Double zMin = Collections.min(zList); //z的最小值

        double sum = 0;
        double zHeigh = 0;//每一个z与最低点的z差值
        for (int i = 0; i < zList.size(); i++) {
            //sum = zList.get(i)+sum;
            zHeigh = zList.get(i) - zMin;
            sum += zHeigh;

        }
        /** v 是总体积
         * v1 最上面的那一部分
         * v2 是中间的圆柱
         * v3 是 圆锥
         *
         * h1 圆锥的高
         * h2 水平地面到圆锥尖的高
         */

        double v1 = 3.14 * (diameter / 2) * (diameter / 2) * (sum / zList.size());
        double v2 = 3.14 * (diameter / 2) * (diameter / 2) * (zMin - h2 - h1);
        double v3 = (3.14 * (diameter / 2) * (diameter / 2) * h1) / 3;
        double sumV = v1 + v2 + v3;
        BigDecimal bdv = BigDecimal.valueOf(sumV);
        double v = bdv.setScale(2, RoundingMode.HALF_UP).doubleValue();
        return v;
    }

    
    /**
     * 计算实际的点的坐标
     */
    public static String getCoordinates(String rotationxy, double diameter, double high) {
    	
    	StringBuffer pointSb = new StringBuffer();
    	pointSb.append("[");
    	
    	String pointString = null;//所有的点
    	String xyz = null;//每个点的坐标

        double AC = high;
        //double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面的点到设备的水平距离
        double x = 0; //实际点的坐标的x轴
        double y = 0; //实际点的坐标的y轴
        double z = 0; //实际点的坐标的z轴
        int x1 = 0;
        int y1 = 0;
        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离即设备和粮面间两点的距离
        BigDecimal baseNumber = new BigDecimal(basicData);

        //  "18,7,17.88 ; 19,6,28.88 ; 17,4,39.88"  垂直角度，水平角度，返回距离 ;
        String[] split = rotationxy.split(";");
        for (int i = 0; i < split.length; i++) {
            //System.out.println("split[i]是:"+split[i]);
            String[] split1 = split[i].split(",");
            verticalAngle = new BigDecimal(split1[0]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            levelAngle = new BigDecimal(split1[1]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            realHypotenuse = new BigDecimal(split1[2]).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).doubleValue(); //单位转化为米

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            //垂直角度的sin
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinVerticalAngle = new Double(verticalAngleS);
            //垂直角度的cos
            String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
            double cosVerticalAngle = new Double(verticalAngleC);
            
            //竖直三角形的底边 即 平面三角形的斜边  sinVerticalAngle有可能是负值 
            A1B1 = realHypotenuse * sinVerticalAngle; 

            //将水平的角度转化为弧度
            double levelAngleRadians = Math.toRadians(levelAngle);
            //水平角度的sin
            String sinLevel = String.format("%.2f%n", Math.sin(levelAngleRadians));
            double sinLeveld = new Double(sinLevel);
            //水平角度的cos
            String cosLevel = String.format("%.2f%n", Math.cos(levelAngleRadians));
            double cosLeveld = new Double(cosLevel);

            /*
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             * */
            //打到墙上 最大的边
            double maxLength = sinLeveld * diameter;

            if (Math.abs(A1B1)< Math.abs(maxLength)) {
                //实际打在粮面上的点的坐标（此时是以设备为坐标系的原点 分为上下两部分）
                double A1C = realHypotenuse * cosVerticalAngle;
                z = AC - A1C;
                x = Math.abs(A1B1 * sinLeveld);
                if(levelAngle<0){//反方向转的
                  y = -Math.abs(A1B1 * cosLeveld);
                }else{
                  y = Math.abs(A1B1 * cosLeveld);
                }
                //给三维的是 整个圆都在第一象限   则将其纵坐标上移 半径 的单位 横坐标不变
                y = y + diameter/2;
                x1 = (int) Math.round(x);
                y1 = (int) Math.round(y);
                z = (double) (Math.round(z*100)/100);
                xyz = "["+x1+","+y1+","+z+"]"+",";
                
                pointSb.append(xyz);
                //System.out.println("["+x+","+y+","+z+"]"+",");
            }
        }
        pointString = pointSb.substring(0, pointSb.toString().length()-1).toString()+"]";
        return pointString;
    	
    }
    
    /**
     * 有效的z的集合
     */
    public static List<Double> getValidZ(String rotationxy, double diameter, double high){
    	List<Double> zList  = new ArrayList<Double>();
    	
        double AC = high;
        //double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面的点到设备的水平距离
        double z = 0; //实际点的坐标的z轴
        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离即设备和粮面间两点的距离
        BigDecimal baseNumber = new BigDecimal(basicData);

        //  "18,7,17.88 ; 19,6,28.88 ; 17,4,39.88"  垂直角度，水平角度，返回距离 ;
        String[] split = rotationxy.split(";");
        for (int i = 0; i < split.length; i++) {
            //System.out.println("split[i]是:"+split[i]);
            String[] split1 = split[i].split(",");
            verticalAngle = new BigDecimal(split1[0]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            levelAngle = new BigDecimal(split1[1]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            realHypotenuse = new BigDecimal(split1[2]).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).doubleValue(); //单位转化为米

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            //垂直角度的sin
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinVerticalAngle = new Double(verticalAngleS);
            //垂直角度的cos
            String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
            double cosVerticalAngle = new Double(verticalAngleC);
            
            //竖直三角形的底边 即 平面三角形的斜边  sinVerticalAngle有可能是负值 
            A1B1 = realHypotenuse * sinVerticalAngle; 

            //将水平的角度转化为弧度
            double levelAngleRadians = Math.toRadians(levelAngle);
            //水平角度的sin
            String sinLevel = String.format("%.2f%n", Math.sin(levelAngleRadians));
            double sinLeveld = new Double(sinLevel);
            //水平角度的cos
            /*String cosLevel = String.format("%.2f%n", Math.cos(levelAngleRadians));
            double cosLeveld = new Double(cosLevel);*/

            /*
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             * */
            //打到墙上 最大的边
            double maxLength = sinLeveld * diameter;

            if (Math.abs(A1B1)< Math.abs(maxLength)) {
                //实际打在粮面上的点的坐标（此时是以设备为坐标系的原点 分为上下两部分）
                double A1C = realHypotenuse * cosVerticalAngle;
                z = AC - A1C;
                zList.add(z);
            }
        }
        return zList;
    
    }
    
    
    /*public static void main(String[] args) {

    	String rotationxy = "550,4500,12510;500,4050,12513;400,4500,12516;-500,-4050,6542;650,4500,12523;600,3950,12520;500,3600,12524;400,3650,12493;350,4500,12477;-400,-3650,7658;-500,-3600,6589;-600,-3950,5692;700,4500,12522;700,4000,12522;650,3550,12523;550,3150,12523;400,3000,12488;300,3300,12444;250,4500,12398;-300,-3300,9077;-400,-3000,7688;-550,-3150,6087;-650,-3550,5323;-700,-4000,5003;800,4500,12521;800,4050,12522;750,3650,12524;700,3200,12524;600,2850,12524;500,2550,12514;350,2400,12458;250,2700,12377;150,4100,12290;-200,-3050,11714;-300,-2450,9415;-450,-2450,7153;-550,-2750,6130;-650,-3100,5341;-700,-3500,5010;-750,-3900,4734;850,4500,12523;850,4100,12522;850,3700,12522;800,3300,12521;700,2900,12523;650,2500,12526;550,2150,12514;400,1850,12476;300,1700,12423;150,1950,12283;100,4500,12249;-150,-1950,11985;-300,-1700,9426;-400,-1850,7802;-550,-2150,6150;-650,-2500,5350;-700,-2900,5019;-800,-3300,4496;-850,-3700,4268;-850,-4100,4268;950,4500,12532;950,4150,12532;900,3750,12538;900,3400,12537;850,3000,12539;750,2600,12521;700,2250,12523;600,1850,12524;500,1500,12511;350,1150,12457;250,750,12374;150,350,12297;0,0,12120;-150,-350,11988;-250,-750,10584;-350,-1150,8573;-500,-1500,6678;-600,-1850,5739;-700,-2250,5042;-750,-2600,4767;-850,-3000,4268;-900,-3400,3881;-900,-3750,3881;-950,-4150,3661;500,4500,12475";
    	double diameter = 12;
    	double high = 35;
    	
    	
    	List<Double> zList  = new ArrayList<Double>();
    	
        double AC = high;
        //double B1C = 0;//设备和粮面点之间的距离
        double A1B1 = 0;//粮面的点到设备的水平距离
        double z = 0; //实际点的坐标的z轴
        double verticalAngle = 0;//垂直角度
        double levelAngle = 0;//水平角度
        double realHypotenuse = 0;//返回的距离即设备和粮面间两点的距离
        BigDecimal baseNumber = new BigDecimal(basicData);

        //  "18,7,17.88 ; 19,6,28.88 ; 17,4,39.88"  垂直角度，水平角度，返回距离 ;
        String[] split = rotationxy.split(";");
        for (int i = 0; i < split.length; i++) {
            //System.out.println("split[i]是:"+split[i]);
            String[] split1 = split[i].split(",");
            verticalAngle = new BigDecimal(split1[0]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            levelAngle = new BigDecimal(split1[1]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
            realHypotenuse = new BigDecimal(split1[2]).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).doubleValue(); //单位转化为米

            //将垂直的角度转化为弧度
            double verticalAngleRadians = Math.toRadians(verticalAngle);
            //垂直角度的sin
            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
            double sinVerticalAngle = new Double(verticalAngleS);
            //垂直角度的cos
            String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
            double cosVerticalAngle = new Double(verticalAngleC);
            
            //竖直三角形的底边 即 平面三角形的斜边  sinVerticalAngle有可能是负值 
            A1B1 = realHypotenuse * sinVerticalAngle; 

            //将水平的角度转化为弧度
            double levelAngleRadians = Math.toRadians(levelAngle);
            //水平角度的sin
            String sinLevel = String.format("%.2f%n", Math.sin(levelAngleRadians));
            double sinLeveld = new Double(sinLevel);
            //水平角度的cos
            String cosLevel = String.format("%.2f%n", Math.cos(levelAngleRadians));
            double cosLeveld = new Double(cosLevel);

            
             * 对比实际的A1B1与打到墙上时最大的这个边进行比较 如果A1B1小于这个最大边则可取
             * 如果A1B1大于或等于这个最大的边 则打到了墙上 舍去
             * 
            //打到墙上 最大的边
            double maxLength = sinLeveld * diameter;

            if (Math.abs(A1B1)< Math.abs(maxLength)) {
                //实际打在粮面上的点的坐标（此时是以设备为坐标系的原点 分为上下两部分）
                double A1C = realHypotenuse * cosVerticalAngle;
                z = AC - A1C;
                double secondH = 35 - z;
                System.out.println(z+"**********"+A1C+"********"+secondH);
                //zList.add(z);
            }
        }
       
    
    
	}*/
}
