package com.unissoft.interaction.algorithm;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
/**
 * 平房仓
 * @author My Sunshine
 *
 */
public class SquareAlg {

	/**
	 * 定义常量  减速机传动比
	 */
	public static int basicData = 50;
	
	//0.5米做安全隔墙
	public static double boundaryValue = 0.5;
	
	//每隔2米打一个点
	public static double spacingDistance = 2;
	
	public static String getAveragePosition(double houseWideOn, double houseWideUnder, double houseLengthLeft, double houseLengthRight,double houseHigh) {
		 
			StringBuffer sb = new StringBuffer();   
			String str = "";
			//AC 是设备到地面/粮面最高点的高
			double AC = houseHigh;
			/**
			 * 设备安装的点为o 设备到左面的点为a 上面的点为b 右面的点为c 下面的点为d
			 * oa ob oc od 根据设备可以获取得到 
			 * 以o为起点每间隔两米打一个点
			 * 		b
			 * |---|-----------|
			 * | 1 |	2	   |
			 * |   |		   |
			 a |---|o----------|c
			 * | 3 |	4	   |
			 * |---------------|
			 * 		d
			 * 水平转动的方向 以bd为开始 向左即（1和4 区域的）是反方向的；（23区域）是正方向的
			 * 垂直转动的方向 以ac为开始 向上即（1和2区域的）是正方向;(3和4区域)是反方向的
			 * 
			 */
			double oa = houseLengthLeft;
			double ob = houseWideOn;
			double oc = houseLengthRight;
			double od = houseWideUnder;
			/**
			 * 第一块
			 * 以o为起点 oa、ob方向每隔spacingDistance米打一个点 
			 * +1是把起点o算上
			 * 打出来的点 包括区域里的和x轴和y轴
			 */
			double x = 0;//点的横坐标
			double y = 0;//点的纵坐标
			double AB = 0; //理想状态 垂直三角形的底边AB 实际是平面三角形的斜边
		    double level_angle = 0;//水平转动的角度
		    double vertical_angle = 0;//垂直转动角度
		    double BC = 0; //设备和粮面点之间的距离(垂直三角形的斜边)
		    
		    int level_angle1 = 0;//水平转动的角度
		    int vertical_angle1 = 0;//垂直转动角度
			for (int i = 0; i < Math.floor(ob/spacingDistance)+1 ; i++) {//i和j表示点的个数
			  for(int j = 0; j < Math.floor(oa/spacingDistance)+1 ; j++){
				  x = -spacingDistance * j;
				  y = spacingDistance * i;
				  //System.out.println("("+x+","+y+")");
				  AB = Math.sqrt(x*x+y*y);
				  if(x == 0 && y >= 0){//y轴的正半轴以及原点
					  level_angle = 0;
				  }else if(y == 0 && x != 0){//x轴的负半轴
					  level_angle = -90;
				  }else{
					  level_angle = -Math.toDegrees(Math.atan((Math.abs(x))/y));
				  }
				  BC = Math.sqrt(AB*AB+AC*AC);
				  vertical_angle = Math.toDegrees(Math.acos(AC/BC));
				  
				  level_angle1 =  (int) (Math.round(level_angle) * basicData);
				  vertical_angle1 =  (int) (Math.round(vertical_angle) * basicData);
				  
				  str = vertical_angle1 + ","+level_angle1+";";
				  sb.append(str);
			  }
			  
			}
			
			/**
			 * 第二块
			 * 以o为起点 ob、oc方向每隔两米打一个点 
			 * 除去第一块的ob上的点 所以 外循环+1
			 * 
			 * 打出来的点 只有 x轴和区域的点
			 */
			for(int i = 0; i < Math.floor(ob/spacingDistance)+1 ; i++){
				for(int j = 1; j <= Math.floor(oc/spacingDistance) ; j++){
					  x = spacingDistance * j;
					  y = spacingDistance * i;
					  //System.out.println("("+x+","+y+")");
					  AB = Math.sqrt(x*x+y*y);
					  if(y == 0){//x轴的正半轴
						  level_angle = 90;
					  }else{ 
						  level_angle = Math.toDegrees(Math.atan(x/y));
					  }
					  BC = Math.sqrt(AB*AB+AC*AC);
					  vertical_angle = Math.toDegrees(Math.acos(AC/BC));
					  
					  level_angle1 = (int) Math.round(level_angle) * basicData;
					  vertical_angle1 = (int) Math.round(vertical_angle) *basicData;
					  str = vertical_angle1 + ","+level_angle1+";";
					  sb.append(str);
				}
				
			}
			
			/**
			 * 第三块
			 * 以o为起点 oa、od方向每隔两米打一个点 
			 * 
			 * 打出来的点 包括区域里的和y轴
			 */
			
			for (int i = 1; i < Math.floor(od/spacingDistance)+1 ; i++) {
				  for(int j = 0; j < Math.floor(oa/spacingDistance)+1 ; j++){
				  x = -spacingDistance * j;
				  y = -spacingDistance * i;
				  //System.out.println("("+x+","+y+")");
				  AB = Math.sqrt(x*x+y*y);
				  if(x == 0){//y轴的负半轴
					  level_angle = 0;
				  }else{
					  level_angle = Math.toDegrees(Math.atan((Math.abs(x))/y));
				  }
				  BC = Math.sqrt(AB*AB+AC*AC);
				  vertical_angle = -Math.toDegrees(Math.acos(AC/BC));
				  level_angle1 = (int) Math.round(level_angle) * basicData;
				  vertical_angle1 = (int) Math.round(vertical_angle) * basicData;
				  str = vertical_angle1 + ","+level_angle1+";";
				  sb.append(str);
			  }
			  
			}
			
			/**
			 * 第四块
			 * 以o为起点 oc、od方向每隔两米打一个点 
			 * 打出来的点 只有区域的点
			 */
			for(int i = 1; i <= Math.floor(oc/spacingDistance) ; i++){
				for(int j = 1; j <= Math.floor(od/spacingDistance) ; j++){
					  x = spacingDistance * i;
					  y = -spacingDistance * j;
					 // System.out.println("("+x+","+y+")");
					  AB = Math.sqrt(x*x+y*y);
					  level_angle = -Math.toDegrees(Math.atan(x/(Math.abs(y))));
					  BC = Math.sqrt(AB*AB+AC*AC);
					  vertical_angle = -Math.toDegrees(Math.acos(AC/BC));
					  level_angle1 = (int) Math.round(level_angle) * basicData;
					  vertical_angle1 = (int) Math.round(vertical_angle) * basicData;
					  str = vertical_angle1 + ","+level_angle1+";";
					  sb.append(str);
				}
				
			}
			String returnString = sb.substring(0, sb.length()-1);
			//System.out.println(returnString);

		 return returnString;
		 
	 }
	 
	 
	   /**
	     * 计算  z的有效点
	     *
	     * @param rotationxy
	     * @return
	     */
	    public static List<Double> zHeight(String rotationxy,double houseWideOn,double houseWideUnder,double houseLengthLeft,double houseLengthRight,double houseHight) {

			List<Double> zList = new ArrayList<Double>();
			
			//垂直角度
			double verticalAngle = 0;
			//水平角度
	        double levelAngle = 0;
	        //返回的距离
	        double realHypotenuse = 0;
	        //设备到地面的高
	        //double AC = 18;
	        //设备和粮面点之间的距离
	        //double B1C = 0;
	        //粮面到设备的水平距离
	        double A1B1 = 0;
	        //实际点的坐标的z轴
	        //double z = 0; 
	        //真实的点的横坐标的距离
	        double x = 0;
	        //真实的点的纵坐标的距离
	        double y = 0;
	        //加一个边界值后的横纵坐标
	        double x1 = 0;
	        double y1 = 0;
	       //以粮面为基准面的高
	        //double zHigh = 0;
	        
	        double oa = houseLengthLeft; //x的左半轴(长)
    		double ob = houseWideOn; //y的上半轴(宽)
    		double oc = houseLengthRight; //x的右半轴(长)
    		double od = houseWideUnder; //y的下半轴(宽)
    		
	        //基数 
	        BigDecimal baseNumber = new BigDecimal(basicData);
	        
			String[] split = rotationxy.split(";");
	        for (int i = 0; i < split.length; i++) {
	            //System.out.println("split[i]是:"+split[i]);
	            String[] split1 = split[i].split(",");
	            verticalAngle = new BigDecimal(split1[0]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
	            levelAngle = new BigDecimal(split1[1]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
	            realHypotenuse = new BigDecimal(split1[2]).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).doubleValue(); //单位转化为米
	            
	            /***
	             * 去掉打在墙上的点(计算正式的点的坐标，横纵坐标只要有一个>(边-0.5)即为墙上的点)
	             */
	            
	            //将垂直的角度转化为弧度
	            double verticalAngleRadians = Math.toRadians(verticalAngle);
	            //垂直角度的sin
	            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
	            double sinVerticalAngle = new Double(verticalAngleS);
	            //垂直角度的cos
	            String verticalC = String.format("%.2f%n", Math.cos(Math.abs(verticalAngleRadians)));
	            double cosVertical = new Double(verticalC);
	            
	            //将水平的角度转化为弧度
	            double levelAngleRadians = Math.toRadians(levelAngle);
	            //水平角度的sin
	            String levelSinStr = String.format("%.2f%n", Math.sin(Math.abs(levelAngleRadians)));
	            double sinLevelAngle = new Double(levelSinStr);
	            //水平角度的cos
	            String levelCosStr = String.format("%.2f%n", Math.cos(Math.abs(levelAngleRadians)));
	            double cosLevelAngle = new Double(levelCosStr);
	            
	            //平面三角形的斜边
	            A1B1 = realHypotenuse * sinVerticalAngle;
	            
	            //横坐标的距离(只有长度)
	            x = Math.abs(A1B1 * cosLevelAngle ) ;
	            //纵坐标的距离(只有长度)
	            y = Math.abs(A1B1 * sinLevelAngle ) ;
	            
	            //判断真实的坐标与长宽的距离
	            //四面都减少0.5米做安全隔墙 只要不在这范围的都算打在墙上(真实的坐标的距离都加0.5小于实际的边即可)
	            
	    		//加0.5后的横纵坐标的距离
	    		x1 = x + boundaryValue;
	    		y1 = y + boundaryValue;
	    		
	    		 
	             //A1C 是粮面点到设备的垂直距离
	             double A1C = realHypotenuse * cosVertical; 
	             double z = houseHight - A1C;
	             
	            if(levelAngle < 0 && verticalAngle>0 ){
	            	//第一区域
	            	if(x1<oa && y1<ob){
	                     zList.add(z);
	            	}
	            }else if(levelAngle > 0 && verticalAngle>0){
	            	//第二区域
	            	if(x1<oc && y1<ob){
	                    zList.add(z);
	           	    }
	            	
	            }else if(levelAngle > 0 && verticalAngle<0){
	            	//第三区域
	            	if(x1<oa && y1<od){
	                    zList.add(z);
	           	    }
	            	
	            }else if(levelAngle < 0 && verticalAngle<0){
	            	//第四区域
	            	if(x1<oc && y1<od){
	                    zList.add(z);
	           	    }
	            }else if(levelAngle == 0 || levelAngle == -90 || levelAngle == 90){
	            	//坐标轴上的点
	            	if(verticalAngle>0){
	            		if(y1<ob){
	            			zList.add(z);
	            		}
	            	}else if(verticalAngle<0){
	            		if(y1<od){
	            			zList.add(z);
	            		}
	            	}
	            }
	            
	        }
	        
	        return zList;
		
	    }
	    
	    
	    /**
	     * 计算体积---------------
	     *
	     * @param zList 粮面高集合(z值的集合)
	     */
	    public static double calculateVolume(List<Double> zList,double houseLength,double houseWide,double houseHight) {
	    	
	    	//17.1  18.3  19.2  18.3    20.1  16.4  19.8  16.7  19.0  18.9
	        Double zMin = Collections.min(zList); //z的最小值

	        double sum = 0;
	        double zHeigh = 0;//每一个z与最低点的z差值
	        for (int i = 0; i < zList.size(); i++) {
	            //sum = zList.get(i)+sum;
	            zHeigh = zList.get(i) - zMin;
	            sum += zHeigh;
	        }
	        /** v 是总体积
	         * v1 最上面的那一部分不规则的体积
	         * v2 是长方体的体积
	         * 
	         */

	        double v1 = houseLength * houseWide * (sum / zList.size());
	        double v2 = houseLength * houseWide * houseHight;
	        double sumV = v1 + v2;
	        BigDecimal bdv = BigDecimal.valueOf(sumV);
	        double v = bdv.setScale(2, RoundingMode.HALF_UP).doubleValue();
	    	return v;
	    }
	    
	    

	    /**
	     * 计算实际的点的坐标
	     */
	    public static String getCoordinates(String rotationxy, double houseWideOn,double houseWideUnder,double houseLengthLeft,double houseLengthRight, double high) {
	    	
	    	StringBuffer pointSb = new StringBuffer();
	    	pointSb.append("[");
	    	
	    	String pointString = null;//所有的点
	    	String xyz = null;//每个点的坐标

	        double AC = high;
	        //double B1C = 0;//设备和粮面点之间的距离
	        double A1B1 = 0;//粮面的点到设备的水平距离
	        double x = 0; //实际点的坐标的x轴
	        double y = 0; //实际点的坐标的y轴
	        double z = 0; //实际点的坐标的z轴
	        //加一个边界值后的横纵坐标
	        double x1 = 0;
	        double y1 = 0;
	        //给三维的坐标
	        int x2 = 0;
	        int y2 = 0;
	        double verticalAngle = 0;//垂直角度
	        double levelAngle = 0;//水平角度
	        double realHypotenuse = 0;//返回的距离即设备和粮面间两点的距离
	        
	        double oa = houseLengthLeft; //x的左半轴(长)
    		double ob = houseWideOn; //y的上半轴(宽)
    		double oc = houseLengthRight; //x的右半轴(长)
    		double od = houseWideUnder; //y的下半轴(宽)
    		
	        BigDecimal baseNumber = new BigDecimal(basicData);

	        //  "18,7,17.88 ; 19,6,28.88 ; 17,4,39.88"  垂直角度，水平角度，返回距离 ;
	        String[] split = rotationxy.split(";");
	        for (int i = 0; i < split.length; i++) {
	            //System.out.println("split[i]是:"+split[i]);
	            String[] split1 = split[i].split(",");
	            verticalAngle = new BigDecimal(split1[0]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
	            levelAngle = new BigDecimal(split1[1]).divide(baseNumber, 0, RoundingMode.HALF_UP).doubleValue();
	            realHypotenuse = new BigDecimal(split1[2]).divide(new BigDecimal(1000), 2, RoundingMode.HALF_UP).doubleValue(); //单位转化为米

	            //将垂直的角度转化为弧度
	            double verticalAngleRadians = Math.toRadians(verticalAngle);
	            //垂直角度的sin
	            String verticalAngleS = String.format("%.2f%n", Math.sin(verticalAngleRadians));
	            double sinVerticalAngle = new Double(verticalAngleS);
	            //垂直角度的cos
	            String verticalAngleC = String.format("%.2f%n", Math.cos(verticalAngleRadians));
	            double cosVerticalAngle = new Double(verticalAngleC);
	            
	            //将水平的角度转化为弧度
	            double levelAngleRadians = Math.toRadians(levelAngle);
	            //水平角度的sin
	            String sinLevel = String.format("%.2f%n", Math.sin(levelAngleRadians));
	            double sinLeveld = new Double(sinLevel);
	            //水平角度的cos
	            String cosLevel = String.format("%.2f%n", Math.cos(levelAngleRadians));
	            double cosLeveld = new Double(cosLevel);

	            //竖直三角形的底边 即 平面三角形的斜边   
	            A1B1 = realHypotenuse * sinVerticalAngle; 

	            //横坐标的距离(只有长度)
	            x = Math.abs(A1B1 * cosLeveld ) ;
	            //纵坐标的距离(只有长度)
	            y = Math.abs(A1B1 * sinLeveld ) ;
	            
	            //判断真实的坐标与长宽的距离
	            //四面都减少0.5米做安全隔墙 只要不在这范围的都算打在墙上(真实的坐标的距离都加0.5小于实际的边即可)
	            
	    		//加0.5后的横纵坐标的距离
	    		x1 = x + boundaryValue;
	    		y1 = y + boundaryValue;
	    		
	    		 
	             //A1C 是粮面点到设备的垂直距离
	             double A1C = realHypotenuse * cosVerticalAngle; 
	             z = high - A1C;
	            
	             /*
	              *  给三维的坐标是 1234 模块都属于第一象限 则需要将原点移动至仓房的左下角
	              *  
	              *  坐标移动 横坐标向左移动oa的长度 则＋oa(长度) 纵坐标向下移动od的长度 则＋od(长度)
	              */
	             
	                x2 = (int) Math.round(x1+oa);
	                y2 = (int) Math.round(y1+od);
	                z = (double) (Math.round(z*100)/100);
            		xyz = "["+x2+","+y2+","+z+"]"+",";

	             if(levelAngle < 0 && verticalAngle>0 ){
		            	//第一区域
		            	if(x1<oa && y1<ob){
		            		pointSb.append(xyz);
		            	}
		            }else if(levelAngle > 0 && verticalAngle>0){
		            	//第二区域
		            	if(x1<oc && y1<ob){
		            		pointSb.append(xyz);
		           	    }
		            	
		            }else if(levelAngle > 0 && verticalAngle<0){
		            	//第三区域
		            	if(x1<oa && y1<od){
		            		pointSb.append(xyz);
		           	    }
		            	
		            }else if(levelAngle < 0 && verticalAngle<0){
		            	//第四区域
		            	if(x1<oc && y1<od){
		            		pointSb.append(xyz);
		           	    }
		            }else if(levelAngle == 0 || levelAngle == -90 || levelAngle == 90){
		            	//坐标轴上的点
		            	if(verticalAngle>0){
		            		if(y1<ob){
		            			pointSb.append(xyz);
		            		}
		            	}else if(verticalAngle<0){
		            		if(y1<od){
		            			pointSb.append(xyz);
		            		}
		            	}
		            }
	        }
	        pointString = pointSb.substring(0, pointSb.toString().length()-1).toString()+"]";
	        return pointString;
	    	
	    }
	    
}
