package com.unissoft.interaction.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.unissoft.interaction.algorithm.SquareAlg;
import com.unissoft.interaction.entity.ReMsg;
import com.unissoft.interaction.entity.STask;
import com.unissoft.interaction.entity.TeData;
import com.unissoft.interaction.mapper.ReMsgMapper;
import com.unissoft.interaction.mapper.STaskMapper;
import com.unissoft.interaction.mapper.TeDataMapper;
import com.unissoft.interaction.service.SquareService;
import com.unissoft.parameter.mapper.HardwareParameterMapper;
import com.unissoft.parameter.model.HardwareParameter;
import com.unissoft.systemManage.mapper.OrgInfoMapper;
import com.unissoft.systemManage.mapper.PointDataMapper;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.model.PointData;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

@Service(value = "squareService")
public class SquareServiceImpl implements SquareService {

    private static final Logger logger = LoggerFactory.getLogger(SquareServiceImpl.class);

    //static String base_url = "http://192.168.0.61:4017/";
    static boolean flag = true;
    /*static double houseWideOn;
    static double houseWideUnder;
    static double houseLengthLeft;
    static double houseLengthRight;*/
    private STaskMapper sTaskMapper;
    private TeDataMapper teDataMapper;
    private ReMsgMapper reMsgMapper;
    
    @Autowired
    public void setsTaskMapper(STaskMapper sTaskMapper) {
        this.sTaskMapper = sTaskMapper;
    }

    @Autowired
    public void setTeDataMapper(TeDataMapper teDataMapper) {
        this.teDataMapper = teDataMapper;
    }

    @Autowired
    public void setReMsgMapper(ReMsgMapper reMsgMapper) {
        this.reMsgMapper = reMsgMapper;
    }
    
    @Resource
    private HardwareParameterMapper hardwareParameterMapper;

    @Resource
    private OrgInfoMapper orgInfoMapper;
    
    @Resource
    private TeDataServiceImpl teDataServiceImpl;
    
    @Resource
    private PointDataMapper pointDataMapper;
    
    

    /**
     *第一次打点
     */
    @Override
    public void scanSquareTask(String cfId,double houseWideOn, double houseWideUnder, double houseLengthLeft, double houseLengthRight,double houseHigh) {
    	logger.info("scanSquareTask{} {},{},{}，{},{}",cfId,houseWideOn, houseWideUnder,houseLengthLeft,houseLengthRight,houseHigh);

    	//根据仓房id 获取设备编号
  		QueryWrapper<HardwareParameter> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("house_id", Integer.parseInt(cfId));
        List<HardwareParameter> hardList = hardwareParameterMapper.selectList(queryWrapper);
        if(hardList.size()>0){
        	STask task = new STask();
            Date date = new Date();
            task.setCtime(date);
            task.setDepict(cfId+"仓打点");
            sTaskMapper.insert(task);
            
            TeData teData = new TeData();
            teData.setEquipmentID(hardList.get(0).getDeviceCode());//设备id-----设备编码
            teData.setTaskID(cfId+","+ task.getId());//拼接一个仓房id
            teData.setStime(date);
            
            //第一次待打点
            String rotationxy = SquareAlg.getAveragePosition(houseWideOn,houseWideUnder,houseLengthLeft,houseLengthRight, houseHigh);
            teData.setRotationxy(rotationxy);
            logger.info("第一次打点数据{}",teData);

            //ip地址
            String base_url = "http://"+hardList.get(0).getIpAdress()+":"+hardList.get(0).getPort()+"/";
            
            //发送给硬件
            postHelper(teData,base_url);
            //第一次打点标识
            flag = true;
        }else{
          	logger.info("该仓房没有硬件参数");
        }
    }
    
    /**
     * 第二次打点
     */
    @Async
    @Override
    public void scanTask(TeData teData) {
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        //只运行 发送一次。
        if (flag) {
            // 第二次待打点

            //根据仓房id查询其信息
        	String taskId = teData.getTaskID();
        	String houseId = taskId.split(",")[0];//仓房id
        	
        	//查询设备的ip和端口
            QueryWrapper<HardwareParameter> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("house_id", Integer.parseInt(houseId));
            List<HardwareParameter> hardList = hardwareParameterMapper.selectList(queryWrapper);
            
            if(hardList.size()>0){
            	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));//--------------------待完
            	
            	/*double houseWideOn = 0;
                double houseWideUnder = 0;
                double houseLengthLeft = 0;
                double houseLengthRight = 0;
                double houseHight = 0;*/
            	
            	double houseWideOn = orgInfoHouse.getHouseWideOn().doubleValue();
         		double houseWideUnder = orgInfoHouse.getHouseWideUnder().doubleValue();
         		double houseLengthLeft = orgInfoHouse.getHouseLengthLeft().doubleValue();
         		double houseLengthRight = orgInfoHouse.getHouseLengthRight().doubleValue();
         		double houseHight = Double.valueOf(orgInfoHouse.getHigh());
                
                
            	//计算以粮面为基准面的高
                List<Double> zList = SquareAlg.zHeight(teData.getRotationxy(),houseWideOn,houseWideUnder,houseLengthLeft,houseLengthRight,houseHight);
                //double zHeight = Collections.max(zList);//纵坐标的最大值
                double zHeight = Collections.min(zList);//纵坐标的最小值
                double secondH = houseHight - zHeight;//仓房的高-点的纵坐标
                logger.info("以粮面为基准面的高secondH {}",secondH);

                //打点
                String rotationxy = SquareAlg.getAveragePosition(houseWideOn,houseWideUnder,houseLengthLeft,houseLengthRight, secondH);
                teData.setRotationxy(rotationxy);
                teData.setStime(new Date());//第二次开始的时间
                
                //IP地址
                String base_url = "http://"+hardList.get(0).getIpAdress()+":"+hardList.get(0).getPort()+"/";
                
                //发送给硬件
                postHelper(teData,base_url);

                //打点次数的标识
                flag = false;
            }
        	
        } else { 
        	//打点两次完毕 计算体积
            double v = this.calculateVolume(teData.getTaskID());
            //增加数据到数据查询表(即体积、重量)
            //Te.addNBasicEdge(v,teData.getTaskID());
            teDataServiceImpl.addNBasicEdge(v, teData.getTaskID());
            //计算实际的点的坐标(用于三维展示)-------------todo
            this.addPoint(teData.getTaskID());
            
        }

    }

    @Override
    public int saveTeData(TeData teData) {
        return teDataMapper.insert(teData);
    }

    public ReMsg postHelper(TeData teData,String base_url) {

        logger.info("in ...... postHelper teData{}",teData);

        RestTemplate restTemplate = new RestTemplate();
        teData.setCurrenttime(new Date());
        ReMsg reMsg1 = restTemplate.postForObject(base_url + "equipmentTestingIsReady", teData, ReMsg.class);
        //reMsgMapper.insert(reMsg1);
        logger.info("postHelper reMsg1 {}",reMsg1);

        if (reMsg1 != null) {
            if ("0".equals(reMsg1.getCode())) {
                ReMsg reMsg2 = restTemplate.postForObject(base_url + "equipmentTestingStart", teData, ReMsg.class);
                logger.info("postHelper reMsg2 {}",reMsg2);
                reMsgMapper.insert(reMsg2);
                return reMsg2;
            }
        }
        logger.info("postHelper post task_ {}",teData);

        return null;
    }

    @Override
    public double calculateVolume(String taskID) {
    	
    	
    	//（第一次扫描点数总和+第二次扫描点数总和）*（仓房长*宽/(第一次扫描点数+第二次扫描点数)）=体积
       /* double finalLsTj = (finalSumZ + finalSumZCopy) * (smExtent * smWide / (byOperateIdCOUNT + byOperateIdCOUNTCopy));
        double finalLsZl = wd * finalLsTj / 1000 / 1000 * 1000;
        */
        
        // TODO Auto-generated method stub
        //List<Double> zList = new ArrayList<Double>();
        //String[] split = null;
        String str = "";

        QueryWrapper<TeData> queryWrapper = new QueryWrapper<>();

        List<TeData> teDataList = null;
        queryWrapper.eq("taskID", taskID);
        teDataList = teDataMapper.selectList(queryWrapper);
        for (TeData teData : teDataList) {
        	str += teData.getRotationxy()+";";
        }
        //两次打点的值
        String xyStr = str.substring(0, str.length()-1);
        
        //根据仓房id查询其信息
    	String houseId = taskID.split(",")[0];//仓房id
    	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));//--------------------
    	
    	/*double houseWideOn = 0;
        double houseWideUnder = 0;
        double houseLengthLeft = 0;
        double houseLengthRight = 0;
        double houseHight = 0;*/
    	
    	double houseWideOn = orgInfoHouse.getHouseWideOn().doubleValue();
 		double houseWideUnder = orgInfoHouse.getHouseWideUnder().doubleValue();
 		double houseLengthLeft = orgInfoHouse.getHouseLengthLeft().doubleValue();
 		double houseLengthRight = orgInfoHouse.getHouseLengthRight().doubleValue();
 		double houseLength = orgInfoHouse.getChang().doubleValue();
 		double houseWide = orgInfoHouse.getKuan().doubleValue();
 		double houseHight = Double.valueOf(orgInfoHouse.getHigh());
        
        
        //去除打在墙上的点 得到的集合是有效的点的纵坐标
        List<Double> zList = SquareAlg.zHeight(xyStr,houseWideOn,houseWideUnder,houseLengthLeft,houseLengthRight,houseHight);
        //体积
        double v =SquareAlg.calculateVolume(zList,houseLength,houseWide,houseHight);
        return v;

    }
    
    
    /**
     * 实际打点的坐标
     * @param taskId
     */
    public void addPoint(String taskId){
    	PointData ponitData = new PointData();
    	
    	String houseId = taskId.split(",")[0];//仓房id
        //获取仓房信息
    	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));
		String high = orgInfoHouse.getHigh();//高
		Double h = Double.parseDouble(high);
		double houseWideOn = orgInfoHouse.getHouseWideOn().doubleValue();
 		double houseWideUnder = orgInfoHouse.getHouseWideUnder().doubleValue();
 		double houseLengthLeft = orgInfoHouse.getHouseLengthLeft().doubleValue();
 		double houseLengthRight = orgInfoHouse.getHouseLengthRight().doubleValue();
 		double length = orgInfoHouse.getChang().doubleValue();
 		double wide = orgInfoHouse.getKuan().doubleValue();
    	//查询打点返回的数据
    	StringBuffer sb = new StringBuffer();
    	QueryWrapper<TeData> queryWrapper = new QueryWrapper<>();
        List<TeData> teDataList = null;
        queryWrapper.eq("taskID", taskId);
        teDataList = teDataMapper.selectList(queryWrapper);
        if(teDataList.size()>0){
        	for (TeData teData : teDataList) {
        		sb.append(teData.getRotationxy()+";");
            	
            }
        	String rotationxy = sb.substring(0, sb.length()-1).toString();//发送的角度和返回的距离  150,50,90;200,-100,90;50,-50,90
        	
        	//计算实际的坐标
        	String xyz = SquareAlg.getCoordinates(rotationxy,houseWideOn,houseWideUnder,houseLengthLeft,houseLengthRight , h);
        	
        	ponitData.setHouseId(orgInfoHouse.getId());
        	ponitData.setCreateTime(new Date());
        	ponitData.setLkHeight(high);//保存高
    		ponitData.setPoints(xyz);//保存坐标
    		ponitData.setLength(length+"");//仓房的长
    		ponitData.setWide(wide+"");//仓房的宽
    		
    		//String jsonString = "[[1,2,3],[4,5,62.33],[1,5,2.3]]";
    		JSONArray jsonArray=JSONArray.parseArray(xyz);
    		List<Double> list = new ArrayList<>();
    		
    		for(int i=0;i<jsonArray.size();i++){
    		    double z = Double.parseDouble(jsonArray.get(i).toString().split(",")[2].split("]")[0]);
    		    list.add(z);
    		    //System.out.println(z);
    		}
    		Double max = Collections.max(list);
    		max = (double) (Math.round(max*100)/100);
    	    Double min = Collections.min(list);
    	    min = (double) (Math.round(min*100)/100);
    	    Double minHeigth = (max+min)/2;
    	    ponitData.setVerticalMaxHeigth(max+"");//垂直最大高度
    	    ponitData.setVerticalMinHeigth(min+"");//垂直最小高度
    	    ponitData.setMinHeigth(minHeigth+"");//----------------待确定
    		pointDataMapper.insert(ponitData);
        	
        }
    }

}
