package com.unissoft.interaction.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.unissoft.interaction.algorithm.CylinderAlg;
import com.unissoft.interaction.entity.DensityData;
import com.unissoft.interaction.entity.ReMsg;
import com.unissoft.interaction.entity.STask;
import com.unissoft.interaction.entity.TeData;
import com.unissoft.interaction.mapper.DensityDataMapper;
import com.unissoft.interaction.mapper.ReMsgMapper;
import com.unissoft.interaction.mapper.STaskMapper;
import com.unissoft.interaction.mapper.TeDataMapper;
import com.unissoft.interaction.service.TeDataService;
import com.unissoft.parameter.mapper.HardwareParameterMapper;
import com.unissoft.parameter.model.HardwareParameter;
import com.unissoft.systemManage.mapper.NBasicEdgeMapper;
import com.unissoft.systemManage.mapper.OrgInfoMapper;
import com.unissoft.systemManage.mapper.PointDataMapper;
import com.unissoft.systemManage.model.NBasicEdge;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.model.PointData;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import javax.annotation.Resource;

@Service(value = "teDataService")
public class TeDataServiceImpl implements TeDataService {

    private static final Logger logger = LoggerFactory.getLogger(TeDataServiceImpl.class);

    //static String base_url = "http://192.168.0.61:4017/";
    static boolean flag = true;
    static double h1 = 6; //h1 圆锥的高
    static double h2 = 3; //h2 水平地面到圆锥尖的高
    private STaskMapper sTaskMapper;
    private TeDataMapper teDataMapper;
    private ReMsgMapper reMsgMapper;
    
    @Autowired
    public void setsTaskMapper(STaskMapper sTaskMapper) {
        this.sTaskMapper = sTaskMapper;
    }

    @Autowired
    public void setTeDataMapper(TeDataMapper teDataMapper) {
        this.teDataMapper = teDataMapper;
    }

    @Autowired
    public void setReMsgMapper(ReMsgMapper reMsgMapper) {
        this.reMsgMapper = reMsgMapper;
    }
    
    
    @Resource
    private NBasicEdgeMapper nBasicEdgeMapper;
    @Resource
    private PointDataMapper pointDataMapper;
    @Resource
    private HardwareParameterMapper hardwareParameterMapper;
    @Resource
    private OrgInfoMapper orgInfoMapper;
    @Resource
    private DensityDataMapper densityDataMapper;
    

    /**
     * todo-----------暂时不用
     * http://local.wxlxit.com:61001/
     * 1. equipmentTestingIsReady
     * 2. equipmentTestingStart
     * 3. equipmentTestingEnd
     */
    @Override
    public void scanTask(double diameter, double high) {
    	String base_url = "http://192.168.0.61:4017/";
        logger.info("圆筒仓第一次打点的直径和高: {},{}",diameter,high);
        STask task = new STask();
        Date date = new Date();
        task.setCtime(date);
        task.setDepict("test");
        sTaskMapper.insert(task);

        TeData teData = new TeData();
        teData.setEquipmentID("hljcbk1001");
        teData.setTaskID("" + task.getId());
        teData.setStime(date);

        // 第一次待打点
        String rotationxy = CylinderAlg.getAveragePosition1(diameter, high);
        teData.setRotationxy(rotationxy);
        //int inum = teDataMapper.insert(teData);

        logger.info("scanTask sTaskMapper teDataMapper {},{}",task,teData);

        postHelper(teData,base_url);
        //equipmentTestingEnd
        flag = true;

    }
    
    /**
     * 第一次打点
     * 根据仓房id 获取信息 进行打点
     */
	@Override
	public void scanTask(String cfId, Double diameter, Double h) {
		// TODO Auto-generated method stub
		//根据仓房id 获取设备编号
		QueryWrapper<HardwareParameter> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("house_id", Integer.parseInt(cfId));
        List<HardwareParameter> hardList = hardwareParameterMapper.selectList(queryWrapper);
        if(hardList.size()>0){
        	logger.info("圆筒{}仓第一次打点的直径和高: {},{}",cfId ,diameter,h);
            STask task = new STask();
            Date date = new Date();
            task.setCtime(date);
            task.setDepict("第一次"+cfId+"仓打点");
            sTaskMapper.insert(task);

            TeData teData = new TeData();
            teData.setEquipmentID(hardList.get(0).getDeviceCode());//设备id
            teData.setTaskID(cfId+","+ task.getId());//拼接一个仓房id
            teData.setStime(date);
            //IP地址
            String base_url = "http://"+hardList.get(0).getIpAdress()+":"+hardList.get(0).getPort()+"/";

            // 第一次待打点
            String rotationxy = CylinderAlg.getAveragePosition1(diameter, h);
            teData.setRotationxy(rotationxy);
            logger.info("第一次打点的数据{}",teData);
            
            //发送给硬件
            postHelper(teData,base_url);
           
            flag = true;
            
        }
	}
	
	
	/*
	 * 第二次打点 并计算体积 以及点的坐标
	 * 
	 */
    @Async
    @Override
    public void scanTask(TeData teData) {
        try {
            Thread.sleep(10000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        //只运行 发送一次。
        if (flag) {
           
            // 第二次待打点

            /**
             * 计算以粮面为基准面的高
             * (根据设备id获取仓房 以及对应的直径和高)
             */
        	
        	String taskId = teData.getTaskID();
        	String houseId = taskId.split(",")[0];//仓房id
            
            //查询设备的ip和端口
            QueryWrapper<HardwareParameter> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("house_id", Integer.parseInt(houseId));
            List<HardwareParameter> hardList = hardwareParameterMapper.selectList(queryWrapper);
            
            if(hardList.size()>0){
            	
            	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));
            	String high = orgInfoHouse.getHigh();//仓房的高
        		String radius = orgInfoHouse.getRadius();//仓房的半径
        		Double diameter = Double.parseDouble(radius)*2;//直径
        		Double h = Double.parseDouble(high);
        		
        		List<Double> zList = CylinderAlg.getValidZ(teData.getRotationxy(), diameter, h);
        		double z = Collections.max(zList);//纵坐标的最大值
        		double z1 = Collections.min(zList);//纵坐标的最小值
        		//第二次打点的高
        		//double secondH = h - z;//仓房的高-点的纵坐标
        		double secondH = z;
                logger.info("z {} ，z1 {},diameter {}，secondH {}",z,z1,diameter,secondH);

                //打点
                String rotationxy = CylinderAlg.getAveragePosition1(diameter, secondH);
                teData.setRotationxy(rotationxy);
                teData.setStime(new Date());//第二次开始的时间
                
            	//IP地址
                String base_url = "http://"+hardList.get(0).getIpAdress()+":"+hardList.get(0).getPort()+"/";
                
            	//发送给硬件打点数据
                postHelper(teData,base_url);

                flag = false;
            }
            
            
        } else { 
        	//打点两次完毕 计算体积
        	 logger.info("计算体积。。。。");
            double v = this.calculateVolume(teData.getTaskID());//两次数据
            //double v = this.calculateVolumeByOne(teData.getTaskID());//两次数据中的一条数据
            logger.info("计算体积是:{}",v);
            //增加数据到数据查询表(即体积、重量)
            this.addNBasicEdge(v,teData.getTaskID());
            //计算实际的点的坐标(用于三维展示)
            this.addPoint(teData.getTaskID());
        }

    }

    @Override
    public int saveTeData(TeData teData) {
        return teDataMapper.insert(teData);
    }

    public ReMsg postHelper(TeData teData,String base_url) {

        logger.info("in ...... postHelper teData{}",teData);

        RestTemplate restTemplate = new RestTemplate();
        teData.setCurrenttime(new Date());
        ReMsg reMsg1 = restTemplate.postForObject(base_url + "equipmentTestingIsReady", teData, ReMsg.class);
        logger.info("postHelper reMsg1 {}",reMsg1);

        if (reMsg1 != null) {
            if ("0".equals(reMsg1.getCode())) {
                ReMsg reMsg2 = restTemplate.postForObject(base_url + "equipmentTestingStart", teData, ReMsg.class);
                logger.info("postHelper reMsg2 {}",reMsg2);
                reMsgMapper.insert(reMsg2);
                return reMsg2;
            }
        }
        logger.info("postHelper post task_ {}",teData);

        return null;
    }

    @Override
    public double calculateVolume (String taskID){
    	StringBuffer sb = new StringBuffer();
    	Double v = new Double(0);
    	QueryWrapper<TeData> queryWrapper = new QueryWrapper<>();

        List<TeData> teDataList = null;
        queryWrapper.eq("taskID", taskID);
        teDataList = teDataMapper.selectList(queryWrapper);
        
        if(teDataList.size()>0){
        	for (TeData teData : teDataList) {
        		sb.append(teData.getRotationxy()+";");
            }
        	String rotationxy = sb.substring(0, sb.length()-1).toString();//发送的角度和返回的距离  150,50,90;200,-100,90;50,-50,90
        	
        	//获取直径
        	String houseId = taskID.split(",")[0];//仓房id
        	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));
    		String radius = orgInfoHouse.getRadius();//仓房的半径
    		Double diameter = Double.parseDouble(radius)*2;//直径
    		String high = orgInfoHouse.getHigh();//高
    		Double h = Double.parseDouble(high);
    		
    		//获取有效的z值
        	List<Double> zList = CylinderAlg.getValidZ(rotationxy, diameter, h);
            
    		//计算体积
            v = CylinderAlg.calculateVolume(zList, diameter, h1, h2); //体积
        }
        
       return v;
        
    }
    
    @Override
    public double calculateVolumeByOne (String taskID){
    	StringBuffer sb = new StringBuffer();
    	Double v = new Double(0);
    	QueryWrapper<TeData> queryWrapper = new QueryWrapper<>();

        List<TeData> teDataList = null;
        queryWrapper.eq("taskID", taskID);
        teDataList = teDataMapper.selectList(queryWrapper);
        
        if(teDataList.size()>0){
        	/*for (TeData teData : teDataList) {
        		sb.append(teData.getRotationxy()+";");
            }
        	String rotationxy = sb.substring(0, sb.length()-1).toString();//发送的角度和返回的距离  150,50,90;200,-100,90;50,-50,90
        	*/
        	String rotationxy = teDataList.get(0).getRotationxy();
        	//获取直径
        	String houseId = taskID.split(",")[0];//仓房id
        	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));
    		String radius = orgInfoHouse.getRadius();//仓房的半径
    		Double diameter = Double.parseDouble(radius)*2;//直径
    		String high = orgInfoHouse.getHigh();//高
    		Double h = Double.parseDouble(high);
    		
    		//获取有效的z值
        	List<Double> zList = CylinderAlg.getValidZ(rotationxy, diameter, h);
            
    		//计算体积
            v = CylinderAlg.calculateVolume(zList, diameter, h1, h2); //体积
        }
        
       return v;
        
    }
    
    /**
     * 计算 质量  体积
     * @param lkVolume
     * @param taskId
     */
    public void  addNBasicEdge(double lkVolume,String taskId){
    	NBasicEdge nBasicEdge = new NBasicEdge(); 
    	nBasicEdge.setLkVolume(lkVolume+"");//体积
    	String houseId = taskId.split(",")[0];//仓房id
    	
    	//获取密度
    	QueryWrapper<DensityData> densityQueryWrapper = new QueryWrapper<>();
        List<DensityData> densityDataList = null;
        densityQueryWrapper.eq("house_id", houseId);
        densityDataList = densityDataMapper.selectList(densityQueryWrapper);
        String density = "0"; //密度 千克/立方米
        if(densityDataList.size()>0){
        	density = densityDataList.get(0).getDensity();
        }
		//质量
	    BigDecimal big_density = new BigDecimal(density); 
	    BigDecimal big_lkVolume = new BigDecimal(lkVolume); //保留两位小数的体积
	    BigDecimal quality = big_density.multiply(big_lkVolume).divide(new BigDecimal(1000)).setScale(2, RoundingMode.HALF_UP);
		nBasicEdge.setLkWeight(Double.parseDouble(quality+""));//单位 吨
		
		QueryWrapper<TeData> queryWrapper = new QueryWrapper<>();
        List<TeData> teDataList = null;
        queryWrapper.eq("taskID", taskId);
        teDataList = teDataMapper.selectList(queryWrapper);
        if(teDataList.size()>0){
        	nBasicEdge.setBeginTime(teDataList.get(0).getStime());//开始时间
    		nBasicEdge.setEndTime(teDataList.get(1).getEtime());//结束时间
        }
        
        
        //获取仓房信息
    	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));
    	
    	//获取其库名称
    	OrgInfo orgInfo = orgInfoMapper.selectById(orgInfoHouse.getParentId());
    	
		nBasicEdge.setZsk(orgInfo.getOrgName());
		nBasicEdge.setFk(orgInfo.getId()+"");
		nBasicEdge.setHouseId(houseId);
		nBasicEdge.setCaPolic("2");//不报警
		nBasicEdge.setEquiName(orgInfoHouse.getOrgName());//仓房名称
		
		
		//取上次的数据 如果大于上次的数据 则入库 否则出库
		QueryWrapper<NBasicEdge> nQueryWrapper = new QueryWrapper<>();
		nQueryWrapper.eq("fk", orgInfo.getId());
		nQueryWrapper.orderByDesc("id");
        // 分页对象
        List<NBasicEdge> nList = nBasicEdgeMapper.selectList(nQueryWrapper);
        double v = 0 ;
        if(nList.size()>0){
             v =Double.parseDouble(nList.get(0).getLkVolume());
        }
        
		if(v - lkVolume >0){ //出库
			nBasicEdge.setHStatus("4");//仓房状态1,空仓2,入库3,库存4,出库
		}else if (v - lkVolume <0){//入库
			nBasicEdge.setHStatus("2");
		}
		 nBasicEdgeMapper.insert(nBasicEdge);
		
    }

    
    
	
	/**
	 * 实际打点的坐标 并保存到表里
	 */
    public void addPoint(String taskId){
    	PointData ponitData = new PointData();
    	
    	String houseId = taskId.split(",")[0];//仓房id
        //获取仓房信息
    	OrgInfo orgInfoHouse = orgInfoMapper.selectById(Integer.parseInt(houseId));
		String radius = orgInfoHouse.getRadius();//仓房的半径
		Double diameter = Double.parseDouble(radius)*2;//直径
		String high = orgInfoHouse.getHigh();//高
		Double h = Double.parseDouble(high);
		
    	//查询打点返回的数据
    	StringBuffer sb = new StringBuffer();
    	QueryWrapper<TeData> queryWrapper = new QueryWrapper<>();
        List<TeData> teDataList = null;
        queryWrapper.eq("taskID", taskId);
        teDataList = teDataMapper.selectList(queryWrapper);
        if(teDataList.size()>0){
        	for (TeData teData : teDataList) {
        		sb.append(teData.getRotationxy()+";");
            	
            }
        	String rotationxy = sb.substring(0, sb.length()-1).toString();//发送的角度和返回的距离  150,50,90;200,-100,90;50,-50,90
        	
        	//计算实际的坐标
        	String xyz = CylinderAlg.getCoordinates(rotationxy, diameter, h);
        	
        	ponitData.setHouseId(orgInfoHouse.getId());
        	ponitData.setCreateTime(new Date());
        	ponitData.setLkHeight(high);//保存高
        	ponitData.setRadius(radius);//半径
    		ponitData.setPoints(xyz);//保存坐标
    		
    		//String jsonString = "[[1,2,3],[4,5,62.33],[1,5,2.3]]";
    		JSONArray jsonArray=JSONArray.parseArray(xyz);
    		List<Double> list = new ArrayList<>();
    		
    		for(int i=0;i<jsonArray.size();i++){
    		    double z = Double.parseDouble(jsonArray.get(i).toString().split(",")[2].split("]")[0]);
    		    list.add(z);
    		    //System.out.println(z);
    		}
    		Double max = Collections.max(list);
    		max = (double) (Math.round(max*100)/100);
    	    Double min = Collections.min(list);
    	    min = (double) (Math.round(min*100)/100);
    	    Double minHeigth = (max+min)/2;
    	    ponitData.setVerticalMaxHeigth(max+"");//垂直最大高度
    	    ponitData.setVerticalMinHeigth(min+"");//垂直最小高度
    	    ponitData.setMinHeigth(minHeigth+"");//----------------待确定
    		pointDataMapper.insert(ponitData);
        	
        }
        
    	
    }
    
    public static void main(String[] args) {
    	String xyz = "[[1,2,3000],[4,5,62.330],[1,5,23000]]";
    	JSONArray jsonArray=JSONArray.parseArray(xyz);
    	for(int i=0;i<jsonArray.size();i++){
		    double z = Double.parseDouble(jsonArray.get(i).toString().split(",")[2].split("]")[0])/1000;//单位转化为米
		    //System.out.println(z);
		}
    	
	}

	@Override
	public void tj(String taskID) {
		// TODO Auto-generated method stub
		double v = this.calculateVolume(taskID);
		
		BigDecimal big_density = new BigDecimal(500); 
	    BigDecimal big_lkVolume = new BigDecimal(v); //保留两位小数的体积
	    BigDecimal quality = big_density.multiply(big_lkVolume).divide(new BigDecimal(1000)).setScale(2, RoundingMode.HALF_UP);
		double m = Double.parseDouble(quality+"");//单位 吨
		System.out.println("体积v:"+v+"-----"+"重量是："+m);
		
	}

	@Override
	public void calibration(String cfId, double diameter, double height) {
		// TODO Auto-generated method stub
		//根据仓房id 获取设备编号
				QueryWrapper<HardwareParameter> queryWrapper = new QueryWrapper<>();
		        queryWrapper.eq("house_id", Integer.parseInt(cfId));
		        List<HardwareParameter> hardList = hardwareParameterMapper.selectList(queryWrapper);
		        StringBuffer sb = new StringBuffer();
		        if(hardList.size()>0){
		        	logger.info("圆筒{}仓打点前校准的直径和高: {},{}",cfId ,diameter,height);
		            STask task = new STask();
		            Date date = new Date();
		            task.setCtime(date);
		            task.setDepict(cfId+"仓是否校准");
		            sTaskMapper.insert(task);

		            TeData teData = new TeData();
		            teData.setEquipmentID(hardList.get(0).getDeviceCode());//设备id
		            teData.setTaskID(cfId+","+ task.getId());//拼接一个仓房id
		            teData.setStime(date);
		            //IP地址
		            String base_url = "http://"+hardList.get(0).getIpAdress()+":"+hardList.get(0).getPort()+"/";
		            
		            //打一个点判断是否校准 垂直角度90，水平不变
		            int vertical = CylinderAlg.basicData*90;
		            sb.append(vertical);
		            sb.append(",");
		            sb.append(0);
		            teData.setRotationxy(sb.toString());
		            //发送给硬件
		            postHelper(teData,base_url);

		        }
	}


}
