package com.unissoft.mqConsumer;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.unissoft.mqConsumer.Manager.ServiceManager;
import com.unissoft.parameter.model.HardwareParameter;
import com.unissoft.parameter.service.HardwareParameterService;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

/**
 * ClassName: HardwareParameterConsumer
 * Package: com.unissoft.mqConsumer
 * Description: 硬件参数 消息消费者
 *
 * @Author zlq
 * @Create 2023/3/3 10:42
 * @Version 1.0
 */
@Slf4j
@Component
public class Consumer {
    @Autowired
    private ServiceManager serviceManager;


    private final String topic = "TOPIC_LOCAL";
    //库端TOPIK，云端使用
    private final String topicLocal="TOPIC_LOCAL";
    //云端topic,库端使用
    private final String topicCloud="TOPIC_CLOUD";

    private final String consumerGroup = "consumerGroup";

    @Component
    @RocketMQMessageListener(topic = topicLocal,consumerGroup = "local")
    public class HPSave implements RocketMQListener<String>{
        @Override
        public void onMessage(String message) {
            if (message == null) {
                log.info("消息为空：｛｝",message);
                return;
            }
            log.info("接收到的消息：{}",message);
            JSONObject message1 = JSON.parseObject(message);
            JSONObject headers = message1.getJSONObject("headers");

            String type = headers.getString("type");
            String payload = message1.getString("payload");
            //获取消息所在业务
            String service = headers.getString("service");

            switch (service){
                case "HD_PARAMETER":
                    serviceManager.HDManager(payload,type);
                    break;
                case "BASIC_ENUM":
                    serviceManager.BasicEnumManager(payload,type);
                    break;
                case "NBASIC_EDGE":
                    serviceManager.NBasicEdgeManager(payload,type);
                    break;
                case "ROLE_INFO":
                    serviceManager.RoleInfoManager(payload,type);
                    break;
                case "USER_INFO":
                    serviceManager.UserInfoManager(payload,type);
                    break;
                case "ORG_INFO":
                    serviceManager.OrgInfoManager(payload,type);
                    break;
                default:
                    log.info("没有可用的业务类型！！");
            }
        }
    }

}
