package com.unissoft.mqProducer;


import com.unissoft.common.MessageEnum;
import com.unissoft.common.ServiceEnum;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.producer.SendResult;
import org.apache.rocketmq.spring.core.RocketMQTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.messaging.MessageHeaders;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

/**
 * ClassName: ProService
 * Package: com.unissoft.mqProducer
 * Description: rocketmq生产者
 *
 * @Author zlq
 * @Create 2023/3/2 9:44
 * @Version 1.0
 */
@Slf4j
@Component("producer")
public class Producer {
    //库端TOPIK，库端使用
    private final String topicLocal="TOPIC_LOCAL";
    //云端topic，云端使用
    private final String topicCloud="TOPIC_CLOUD";

    @Autowired
    private RocketMQTemplate rocketMQTemplate;

    /**
     *
     * @param body 消息内容
     * @param type  消息属性
     */
    public void send(Object body, MessageEnum type, ServiceEnum ser){
        Map<String, Object> map = new HashMap<>();
        map.put("type",type);
        map.put("service",ser);
        org.springframework.messaging.Message<Object> message = MessageBuilder.createMessage(body, new MessageHeaders(map));
        rocketMQTemplate.convertAndSend(topicCloud,message);
        log.info("发送消息：{}",message);
    }

    /**
     *
     * @param msgBody  同步发送的消息
     * @param tag  消息类型
     * @return
     */
    public SendResult sendMsg(String msgBody,String tag) {
        SendResult sendResult = rocketMQTemplate.syncSend(topicLocal+tag, MessageBuilder.withPayload(msgBody).build());
        return sendResult;
    }

}
