package com.unissoft.systemManage.controller;


import com.alibaba.fastjson.JSON;
import com.unissoft.basic.model.BasicEnum;
import com.unissoft.common.*;
import com.unissoft.mqProducer.Producer;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.model.OrgInfoChildren;
import com.unissoft.systemManage.service.OrgInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.websocket.server.PathParam;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 组织机构和仓房表 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@RestController
@RequestMapping("/orgInfo")
public class OrgInfoController {
    @Autowired
    private Producer producer;

    private OrgInfoService orgInfoService;

    @Autowired
    public void setRoleInfoController(OrgInfoService orgInfoService) {
        this.orgInfoService = orgInfoService;
    }

    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable String id) {
        OrgInfo orgInfo = orgInfoService.getById(id);
        return ResultView.success(orgInfo);
    }

    @GetMapping("/authority/getTreeList")
    public ResultView getTreeList(@PathParam("orgName") String orgName) {
        List<OrgInfoChildren> listTree = orgInfoService.getTreeList(orgName);
        return ResultView.success(listTree);
    }

    @SystemLog(operModul = "系统管理", operType = "组织机构", operDesc = "新增组织机构")
    @PostMapping("/authority/save")
    public ResultView save(@RequestBody OrgInfo orgInfo) {
        orgInfo.setBuildTime(DateUtils.getDate());
        boolean save = orgInfoService.save(orgInfo);
        if (save) {
            producer.send(orgInfo, MessageEnum.SAVE, ServiceEnum.ORG_INFO);
            return ResultView.success();
        }else
            return ResultView.error();
    }

    @SystemLog(operModul = "系统管理", operType = "组织机构", operDesc = "修改组织机构")
    @PostMapping("/authority/update")
    public ResultView update(@RequestBody OrgInfo orgInfo) {
        orgInfo.setBuildTime(DateUtils.getDate());
        boolean b = orgInfoService.updateById(orgInfo);
        if (b) {
            producer.send(orgInfo, MessageEnum.UPDATE, ServiceEnum.ORG_INFO);
            return ResultView.success();
        }else
            return ResultView.error();
    }

    /**
     * 根据对象查询
     *
     * @param orgInfo 查询对象
     * @return list
     */
    @PostMapping("/authority/getList")
    public ResultView getList(@RequestBody OrgInfo orgInfo) {
        List<OrgInfo> list = orgInfoService.getList(orgInfo);
        return ResultView.success(list);
    }

    @SystemLog(operModul = "系统管理", operType = "组织机构", operDesc = "删除组织机构")
    @GetMapping("/authority/getByIdDelete/{id}")
    public ResultView getByIdDelete(@PathVariable String id) {
        if (orgInfoService.removeById(id)) {
            producer.send(id, MessageEnum.DELETE, ServiceEnum.ORG_INFO);
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    /**
     * 返回map类型 组织机构编码
     * @param orgId 暂时不用
     * @return
     */
    @GetMapping("/authority/getListMap")
    public ResultView getListMap(@PathParam("orgId") String orgId) {
        List<OrgInfo> list = orgInfoService.getOrgMap(orgId);
        Map<Long, String> collect = list.stream().collect(Collectors.toMap(OrgInfo::getId, OrgInfo::getOrgName));
        return ResultView.success(collect);
    }

}
