package com.unissoft.systemManage.controller;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.common.*;
import com.unissoft.mqProducer.Producer;
import com.unissoft.systemManage.model.RoleInfo;
import com.unissoft.systemManage.model.UserInfo;
import com.unissoft.systemManage.service.RoleInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * <p>
 * 用户角色表 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-20
 */
@RestController
@RequestMapping("/roleInfo")
public class RoleInfoController {
    @Resource
    private Producer producer;

    private RoleInfoService roleInfoService;

    @Autowired
    public void setRoleInfoController(RoleInfoService roleInfoService) {
        this.roleInfoService = roleInfoService;
    }

    @GetMapping("/authority/getRoleById/{id}")
    public ResultView getRoleById(@PathVariable String id) {
        RoleInfo roleInfo = roleInfoService.getById(id);
        return ResultView.success(roleInfo);
    }

    @PostMapping("/authority/getRolePage")
    public ResultView getRoleInfoPage(@RequestBody PageParam pageParam) {
        IPage<RoleInfo> page = roleInfoService.getRoleInfoPage(pageParam);
        return ResultView.success(page);
    }

    @SystemLog(operModul = "系统管理", operType = "角色管理", operDesc = "新增角色信息")
    @PostMapping("/authority/saveRoleInfo")
    public ResultView saveRoleInfo(@RequestBody RoleInfo roleInfo) {
        roleInfo.setBulidTime(DateUtils.getDate());
        boolean save = roleInfoService.save(roleInfo);
        if (save) {
            producer.send(roleInfo, MessageEnum.SAVE, ServiceEnum.ROLE_INFO);
            return ResultView.success();
        }else
            return ResultView.error();
    }

    @SystemLog(operModul = "系统管理", operType = "角色管理", operDesc = "修改角色信息")
    @PostMapping("/authority/updateRoleInfo")
    public ResultView updateRoleInfo(@RequestBody RoleInfo roleInfo) {
        roleInfo.setBulidTime(DateUtils.getDate());
        boolean b = roleInfoService.updateById(roleInfo);
        if (b) {
            producer.send(roleInfo, MessageEnum.UPDATE, ServiceEnum.ROLE_INFO);
            return ResultView.success();
        }else
            return ResultView.error();
    }

    @SystemLog(operModul = "系统管理", operType = "角色管理", operDesc = "删除角色信息")
    @GetMapping("/authority/getByIdDeleteRoleInfo/{id}")
    public ResultView getByIdDeleteRoleInfo(@PathVariable String id) {
        boolean  b= roleInfoService.removeById(id);
        if (b) {
            producer.send(id, MessageEnum.DELETE, ServiceEnum.ROLE_INFO);
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

}
