package com.unissoft.systemManage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.common.MyConstant;
import com.unissoft.common.PageParam;
import com.unissoft.common.ResultView;
import com.unissoft.common.SystemLog;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.service.OrgInfoService;
import com.unissoft.systemManage.service.StorehouseClxxService;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * 仓房信息 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-21
 */
@RestController
@RequestMapping("/storehouse")
public class StorehouseController {

    private OrgInfoService orgInfoService;
    private StorehouseClxxService storehouseClxxService;

    @Autowired
    public StorehouseController(StorehouseClxxService storehouseClxxService, OrgInfoService orgInfoService) {
        this.storehouseClxxService = storehouseClxxService;
        this.orgInfoService = orgInfoService;
    }

    @SystemLog(operModul = "仓房信息", operType = "仓房信息列表", operDesc = "支持分页")
    @ApiOperation(value = "仓房信息列表")
    @PostMapping("/getStorehousePage")
    public ResultView getStorehousePage(@RequestBody PageParam pageParam) {
        //IPage<OrgInfo> page = orgInfoService.getOrgInfoPage(pageParam);
        IPage<List<Map<String, Object>>> page = orgInfoService.getStorehousePage(pageParam);
        return ResultView.success(page);
    }

    @SystemLog(operModul = "仓房信息", operType = "仓房信息编辑", operDesc = "初始化页面数据")
    @ApiOperation(value = "仓房信息编辑")
    @GetMapping("/getStorehouseById")
    public ResultView getStorehouseById(@RequestParam int id) {
        Map<String, Object> map = storehouseClxxService.getStorehouseById(id);
        return ResultView.success(map);
    }

    @SystemLog(operModul = "仓房信息", operType = "更新仓房信息列表")
    @ApiOperation(value = "更新仓房信息")
    @RequestMapping(value="/update", produces = MediaType.APPLICATION_JSON_VALUE, method=RequestMethod.POST)
    public ResultView update(String storehouse) {

        boolean b = false;
        try {
            b = orgInfoService.updateStorehouse(storehouse);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (b) {
            return ResultView.success();
        } else {
            return ResultView.error();
        }
    }

    @SystemLog(operModul = "仓房信息", operType = "删除一条仓房信息", operDesc = "同时删除仓房储粮信息")
    @ApiOperation(value = "删除仓房信息")
    @GetMapping("/deleteStorehouse/{id}")
    public ResultView deleteStorehouse(@PathVariable Integer id) {
        boolean flag = storehouseClxxService.deleteStorehouse(id);
        if (flag) {
            return ResultView.success();
        } else {
            return ResultView.error(MyConstant.ID_NULL);
        }
    }

    @SystemLog(operModul = "仓房信息", operType = "删除一条仓房储粮信息", operDesc = "只是删除仓房的一条储粮信息")
    @ApiOperation(value = "删除仓房的一条储粮信息")
    @GetMapping("/deleteStorehouseClxx/{id}")
    public ResultView deleteStorehouseClxx(@PathVariable String id) {
        boolean flag = storehouseClxxService.removeById(id);
        if (flag) {
            return ResultView.success();
        } else {
            return ResultView.error(MyConstant.ID_NULL);
        }
    }
}
