package com.unissoft.systemManage.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.unissoft.common.*;
import com.unissoft.mqProducer.Producer;
import com.unissoft.systemManage.model.UserInfo;
import com.unissoft.systemManage.service.UserInfoService;
import com.unissoft.utils.JWTUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.websocket.server.PathParam;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author root
 * @since 2021-10-19
 */

@Api(tags = "用户管理")

@RestController
@RequestMapping("/userInfo")
public class UserInfoController {

    private UserInfoService userInfoService;
    @Autowired
    private Producer producer;

    @Autowired
    public void setUserInfoController(UserInfoService userInfoService) {
        this.userInfoService = userInfoService;
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getUserById/{id}")
    public ResultView getUserById(@PathVariable String id) {
        UserInfo userPO = userInfoService.getById(id);
        return ResultView.success(userPO);
    }

    @GetMapping("/authority/getUserList")
    public ResultView getUserList(@PathParam("orgId") String orgId,@PathParam("userRole") String userRole) {
        List<UserInfo> list = userInfoService.getUserList(orgId,userRole);
        return ResultView.success(list);
    }


    @ApiOperation(value = "获取用户列表")
    @PostMapping("/authority/getUserPage")
    public ResultView getUserPage(@RequestBody PageParam pageParam) {
        IPage<UserInfo> page = userInfoService.getUserPage(pageParam);
        return ResultView.success(page);
    }

    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "新增用户信息")
    @PostMapping("/authority/save")
    public ResultView saveUser(@RequestBody UserInfo userInfo) {
        userInfo.setBuildTime(DateUtils.getDate());
        userInfo.setUserPassword(MD5Util.getMD5(userInfo.getUserPassword()));
        String s = UuidUtils.getCode();
        userInfo.setUserId(s);
        boolean save = userInfoService.save(userInfo);
        if (save) {
            producer.send(userInfo, MessageEnum.SAVE, ServiceEnum.USER_INFO);
            return ResultView.success();
        }else
            return ResultView.error();
    }

    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "修改用户信息")
    @PostMapping("/authority/update")
    public ResultView updateUser(@RequestBody UserInfo userInfo) {
        userInfo.setBuildTime(DateUtils.getDate());
        userInfo.setUserPassword(MD5Util.getMD5(userInfo.getUserPassword()));
        boolean b = userInfoService.updateById(userInfo);
        if (b) {
            producer.send(userInfo, MessageEnum.UPDATE, ServiceEnum.USER_INFO);
            return ResultView.success();
        }else
            return ResultView.error();
    }

    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "删除用户信息")
    @GetMapping("/authority/deleteById/{id}")
    public ResultView getByIdForUserDelete(@PathVariable String id) {
        if (userInfoService.removeById(id)) {
            producer.send(id, MessageEnum.DELETE, ServiceEnum.USER_INFO);
            return ResultView.success();
        }
        return ResultView.error(MyConstant.ID_NULL);
    }

    @ApiOperation(value = "登录", notes = "单体应用，临时session处理用户信息。")
    @SystemLog(operModul = "系统管理", operType = "用户管理", operDesc = "用户登录")
    @PostMapping("/login")
    public ResultView userLogin(@RequestParam String userId, @RequestParam String userPassword, HttpServletRequest request) {
        try {
            QueryWrapper<UserInfo> wrapper = new QueryWrapper<>();
            if (!StringUtils.isEmpty(userId) && !StringUtils.isEmpty(userPassword)) {
                wrapper.eq("user_id", userId); // userId 为唯一值
                UserInfo one = userInfoService.getOne(wrapper);
                if (!StringUtils.isEmpty(one)) {
                    String md5 = MD5Util.getMD5(userPassword);
                    if (one.getUserPassword().equals(md5)) {
                        request.getSession().setAttribute("userLogin", one);
                        
                        Map<String, String> map = new HashMap<>(); //用来存放payload信息

                        map.put("id",one.getId().toString());
                        map.put("username",one.getUserName());

                        // 生成token令牌
                        Map<String, Object> token = JWTUtil.generateToken(map,one);
                        
                        return ResultView.success(token);
                    }
                } else {
                    return ResultView.error(ResultEnum.CODE_5.getMsg());
                }
            } else {
                return ResultView.error(ResultEnum.CODE_5.getMsg());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return ResultView.error(ResultEnum.CODE_2.getMsg());
    }
    
}
