package com.unissoft.systemManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.common.MessageEnum;
import com.unissoft.common.PageParam;
import com.unissoft.common.ServiceEnum;
import com.unissoft.mqProducer.Producer;
import com.unissoft.systemManage.mapper.NBasicEdgeMapper;
import com.unissoft.systemManage.mapper.PointDataMapper;
import com.unissoft.systemManage.model.LogOperation;
import com.unissoft.systemManage.model.NBasicEdge;
import com.unissoft.systemManage.model.PointData;
import com.unissoft.systemManage.model.SyncV2xyz;
import com.unissoft.systemManage.service.NBasicEdgeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;

import java.math.BigDecimal;
import java.net.URLDecoder;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author root
 * @since 2021-10-22
 */
@Service
public class NBasicEdgeServiceImpl extends ServiceImpl<NBasicEdgeMapper, NBasicEdge> implements NBasicEdgeService  {
	@Resource
	private Producer producer;

    @Resource
    private NBasicEdgeMapper nBasicEdgeMapper;
    @Resource
    private PointDataMapper pointDataMapper;
/*
    @Override
    public List<NBasicEdge> getTreeList(String houseId) {
        QueryWrapper<NBasicEdge> queryWrapper = new QueryWrapper<>();

        List<NBasicEdge> nBasicEdgeList = null;
        // 没有搜索条件时
        if (!StringUtils.isEmpty(houseId)) {
            queryWrapper.eq("house_id", houseId);
            nBasicEdgeList = nBasicEdgeMapper.selectList(queryWrapper);
        }
        return nBasicEdgeList;
    }*/
    @Override
    public IPage<NBasicEdge> getTreeList(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询条件
        QueryWrapper<NBasicEdge> queryWrapper = new QueryWrapper<>();
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
            NBasicEdge operation = JSON.parseObject(condition, NBasicEdge.class);
            // 获取查询条件
            String houseId = operation.getHouseId();
            if (!StringUtils.isEmpty(houseId)) {
                queryWrapper.eq("house_id", houseId);
            }
        }
        queryWrapper.orderByDesc("id");
        // 分页对象
        return nBasicEdgeMapper.selectPage(new Page<>(pageIndex, pageSize), queryWrapper);
    }
    
    
	@Override
	public List<NBasicEdge> getListByOrgId(PageParam pageParam) {
		// TODO Auto-generated method stub
		QueryWrapper<NBasicEdge> queryWrapper = new QueryWrapper<>();
        String condition = pageParam.getCondition();
        if (!StringUtils.isEmpty(condition)) {
        	JSONObject jsStr = JSON.parseObject(condition);
        	String orgId = jsStr.getString("orgId");
        	if (!StringUtils.isEmpty(orgId)) {
                queryWrapper.eq("fk", orgId);
            }
        }
        queryWrapper.orderByDesc("id");
        // 分页对象
        return nBasicEdgeMapper.selectList(queryWrapper);
	}


	@Override
	public int manualSaveNBasicEdge(String nBasicEdgeStr) throws ParseException {
		// TODO Auto-generated method stub
		String[] strArray = nBasicEdgeStr.split(",");
		NBasicEdge nBasicEdge = new NBasicEdge();
		
		SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		for (String str : strArray) {
			
			if(str.contains("开始时间")){
				String startTime = str.split("：",2)[1].replace("/", "-");
				nBasicEdge.setBeginTime(formatter.parse(startTime));
			}else if(str.contains("重量")){
				String lkWeight = str.split(":")[1];
				BigDecimal bigDecimal = new BigDecimal(lkWeight);
				BigDecimal weight = bigDecimal.divide(new BigDecimal("1000"),2,BigDecimal.ROUND_HALF_UP);
				nBasicEdge.setLkWeight(weight.doubleValue());
			}else if(str.contains("结束时间")){
				String endTime = str.split("：",2)[1].replace("/", "-");
				nBasicEdge.setEndTime(formatter.parse(endTime));
			}else if(str.contains("体积")){
				String lkVolume = str.split(":")[1];
				nBasicEdge.setLkVolume(lkVolume);
			}else if(str.contains("是否报警")){
				if(str.split(":")[1].contains("否")){
					nBasicEdge.setCaPolic("2");
				}else{
					nBasicEdge.setCaPolic("1");
				}
			}else if(str.contains("仓房状态")){
				if(str.split(":")[1].contains("入仓")){
					nBasicEdge.setHStatus("2");
				}else if(str.split(":")[1].contains("出仓")){
					nBasicEdge.setHStatus("4");
				}
			}
		}
		nBasicEdge.setZsk("泉州福海粮油工业有限公司");
		nBasicEdge.setFk("43");
		nBasicEdge.setHouseId("44");
		nBasicEdge.setCron("0 0 0 */1 * ?");
		nBasicEdge.setEquiName("3号筒仓");


		int i = nBasicEdgeMapper.insert(nBasicEdge);
		if (i>=0){
			producer.send(nBasicEdge, MessageEnum.SAVE, ServiceEnum.NBASIC_EDGE);
		}
		return i;
	}


	@Override
	public void saveEdgeData(SyncV2xyz syncV2xyz) throws Exception {
		
		System.out.println("volume----"+syncV2xyz.getVolume());
    	System.out.println("xyz-----"+syncV2xyz.getXyz());
    	System.out.println("startTime----"+syncV2xyz.getStartTime());
    	
		Date startTime = syncV2xyz.getStartTime();
		double volume = syncV2xyz.getVolume();
		String xyz_old = URLDecoder.decode(syncV2xyz.getXyz(), "UTF-8");
		String xyz = xyz_old.replace("][", ",");
		
		System.out.println("xyz后-----"+xyz);
		System.out.println("volume后----"+syncV2xyz.getVolume());
    	System.out.println("startTime后-----"+startTime);
		     
		// TODO Auto-generated method stub
		NBasicEdge nBasicEdge = new NBasicEdge(); 
		//体积
		BigDecimal big_volume = new BigDecimal(volume);
		double lkVolume = big_volume.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
		nBasicEdge.setLkVolume(Math.abs(lkVolume)+"");
		
		String density = "500"; //密度 千克/立方米
		//质量
	    BigDecimal big_density = new BigDecimal(density); 
	    BigDecimal big_lkVolume = new BigDecimal(lkVolume); //保留两位小数的体积
	    BigDecimal quality = big_density.multiply(big_lkVolume).divide(new BigDecimal(1000));
		nBasicEdge.setLkWeight(Double.parseDouble(quality+""));
		
		/*SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		Date beginTime = sdf.parse(startTime);*/
		nBasicEdge.setBeginTime(startTime);//开始时间
		Date date = new Date();
		nBasicEdge.setEndTime(date);//结束时间
		nBasicEdge.setZsk("泉州福海粮油工业有限公司");
		nBasicEdge.setFk("43");
		nBasicEdge.setHouseId("44");
		nBasicEdge.setCaPolic("2");//不报警
		nBasicEdge.setEquiName("3号稻谷仓");
		
		
		//取上次的数据 如果大于上次的数据 则入库 否则出库
		QueryWrapper<NBasicEdge> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("fk", 43);
        queryWrapper.orderByDesc("id");
        // 分页对象
        List<NBasicEdge> nList = nBasicEdgeMapper.selectList(queryWrapper);
        double v = 0 ;
        if(nList.size()>0){
             v =Double.parseDouble(nList.get(0).getLkVolume());
        }
        
		if(v - lkVolume >0){ //出库
			nBasicEdge.setHStatus("4");//仓房状态1,空仓2,入库3,库存4,出库
		}else if (v - lkVolume <0){//入库
			nBasicEdge.setHStatus("2");
		}
		 nBasicEdgeMapper.insert(nBasicEdge);
		
		PointData ponitData = new PointData();
		ponitData.setHouseId(44L);
		ponitData.setCreateTime(date);
		ponitData.setPoints(xyz);
		
		//String jsonString = "[[1,2,3],[4,5,62.33],[1,5,2.3]]";
		JSONArray jsonArray=JSONArray.parseArray(xyz);
		List<Double> list = new ArrayList<>();
		
		for(int i=0;i<jsonArray.size();i++){
		    double z = Double.parseDouble(jsonArray.get(i).toString().split(",")[2].split("]")[0]);
		    list.add(z);
		    //System.out.println(jsonArray.get(i)+"*******************"+jsonArray.get(i).toString().split(",")[2].split("]")[0]);
		}
		Double max = Collections.max(list);
	    Double min = Collections.min(list);
	    Double minHeigth = (max+min)/2;
	    ponitData.setVerticalMaxHeigth(max+"");
	    ponitData.setVerticalMinHeigth(min+"");
	    ponitData.setMinHeigth(minHeigth+"");
	    ponitData.setRadius("6");
	    ponitData.setLkHeight("31");
		pointDataMapper.insert(ponitData);
		
	}
}
