package com.unissoft.systemManage.service.impl;

import com.unissoft.systemManage.model.NBasicEdge;
import com.unissoft.systemManage.model.OrgInfo;
import com.unissoft.systemManage.model.QuantityStatistics;
import com.unissoft.systemManage.model.StorehouseClxx;
import com.unissoft.basic.mapper.BasicEnumMapper;
import com.unissoft.basic.model.BasicEnum;
import com.unissoft.common.PageParam;
import com.unissoft.systemManage.mapper.LogOperationMapper;
import com.unissoft.systemManage.mapper.NBasicEdgeMapper;
import com.unissoft.systemManage.mapper.OrgInfoMapper;
import com.unissoft.systemManage.mapper.QuantityStatisticsMapper;
import com.unissoft.systemManage.mapper.StorehouseClxxMapper;
import com.unissoft.systemManage.service.QuantityStatisticsService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;

import java.math.BigDecimal;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Month;
import java.time.Year;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author root
 * @since 2021-10-22
 */
@Service
public class QuantityStatisticsServiceImpl extends ServiceImpl<QuantityStatisticsMapper, QuantityStatistics> implements QuantityStatisticsService {

	private QuantityStatisticsMapper quantityStatisticsMapper;
	
	@Autowired
    private StorehouseClxxMapper storehouseClxxMapper;
	@Autowired
    private OrgInfoMapper orgInfoMapper;
    @Autowired
    private BasicEnumMapper basicEnumMapper;
    @Autowired
    private NBasicEdgeMapper nBasicEdgeMapper;
    

    @Autowired
    public void setQuantityStatisticsService(QuantityStatisticsMapper quantityStatisticsMapper) {
        this.quantityStatisticsMapper = quantityStatisticsMapper;
    }
    
	@Override
	public List<QuantityStatistics> getData(PageParam pageParam) {
		// TODO Auto-generated method stub
		QueryWrapper<QuantityStatistics> ew = new QueryWrapper<>();
		QueryWrapper<QuantityStatistics> ew1 = new QueryWrapper<>();
		String condition = pageParam.getCondition();
		if (!StringUtils.isEmpty(condition)) {
	        
        	JSONObject jsStr = JSON.parseObject(condition);
        	Integer orgId = jsStr.getIntValue("orgId");
        	Date time = jsStr.getDate("time");
    		if(!StringUtils.isEmpty(orgId.toString())){
    			ew.ge("org_id", orgId);
    		}
    		
    		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//注意月份是MM
    		String time1 = sdf.format(time);
    		ew.apply("UNIX_TIMESTAMP(time) = UNIX_TIMESTAMP('"+time1+"')");
    		ew1.apply("UNIX_TIMESTAMP(time) < UNIX_TIMESTAMP('"+time1+"')");
    		ew1.orderByDesc("time");
        }
		
		List<QuantityStatistics> list = quantityStatisticsMapper.selectList(ew); //查询到满足条件所有的数据
		List<QuantityStatistics> list1 = quantityStatisticsMapper.selectList(ew1); 
		for (QuantityStatistics quantityStatistics : list) {
			BigDecimal OutQuantity = new BigDecimal(quantityStatistics.getOutQuantity());
			BigDecimal OutQuantity1 = new BigDecimal(list1.get(0).getOutQuantity());
			BigDecimal outRate = OutQuantity.subtract(OutQuantity1);
			quantityStatistics.setOutRate(outRate.toString());
			
			BigDecimal IntoQuantity = new BigDecimal(quantityStatistics.getIntoQuantity());
			BigDecimal IntoQuantity1 = new BigDecimal(list1.get(0).getIntoQuantity());
			BigDecimal intoRate = IntoQuantity.subtract(IntoQuantity1);
			quantityStatistics.setIntoRate(intoRate.toString());
		}
		return list;
	}

	@Override
	public Map<String, Object> getDataChange(PageParam pageParam) {
		
        
		// TODO Auto-generated method stub
		Map<String, Object> map = new HashMap<>();
		String condition = pageParam.getCondition();
		if (!StringUtils.isEmpty(condition)) {
        	JSONObject jsStr = JSON.parseObject(condition);
        	Integer houseId = jsStr.getIntValue("houseId");
        	String startTime = jsStr.getString("startTime");
        	String endTime = jsStr.getString("endTime");
        	if(null != houseId && !"".equals(houseId)){
        		OrgInfo orgInfo = orgInfoMapper.selectById(houseId);
        		Map<String, Object> parm_map = new HashMap<>();
        	    parm_map.put("main_id", houseId);
        	    List<StorehouseClxx> list = storehouseClxxMapper.selectByMap(parm_map);
        	    
        	    List dayList = new ArrayList<>();
        	    List wightList = new ArrayList<>();
        	    
        	    if(list.size()>0){
        	    	BasicEnum basicEnum = basicEnumMapper.selectById(list.get(0).getPz());
            	    list.get(0).setCfName(orgInfo.getOrgName());//仓房名称
            	    list.get(0).setPzName(basicEnum.getEnumName());//粮食种类
            	    //目前粮食数量
            	    QueryWrapper<NBasicEdge> queryWrapper = new QueryWrapper<>();
            	    queryWrapper.eq("house_id", houseId);
            	    queryWrapper.orderByDesc("id");
            	    List<NBasicEdge> nBasicEdgeList = nBasicEdgeMapper.selectList(queryWrapper);
            	    
            	    QueryWrapper<NBasicEdge> queryWrapper1 = new QueryWrapper<>();
            	    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//注意月份是MM
            	    Calendar calendar = Calendar.getInstance();
            	    Date date = new Date();
            		calendar.setTime(date);
            		calendar.add(Calendar.DAY_OF_MONTH, -1);  // 在当前日基础上-1
            		//System.out.println(sdf.format(calendar.getTime()));
            		queryWrapper1.likeRight("begin_time", sdf.format(calendar.getTime()));
            		queryWrapper1.orderByDesc("id");
            		queryWrapper1.eq("house_id", houseId);
            		List<NBasicEdge> lastList = nBasicEdgeMapper.selectList(queryWrapper1);//前一天扫描的数据
            		
            	    if(nBasicEdgeList.size()>0){
            	    	list.get(0).setLsNumber(nBasicEdgeList.get(0).getLkWeight().toString());
            	    }else{
            	    	list.get(0).setLsNumber("0");
            	    }
            	    if(nBasicEdgeList.size()>1){
            	    	BigDecimal thisNumber = new BigDecimal(nBasicEdgeList.get(0).getLkWeight());
            	    	BigDecimal lastNumber = new BigDecimal(0);
            	    	if(lastList.size()>0){
                	        lastNumber = new BigDecimal(lastList.get(0).getLkWeight());
                	        //当期环比增长（下降）率(%)=[(当期价格/上期价格)-1]*100%。
                	        
                	        BigDecimal changeRatio = (thisNumber.divide(lastNumber,2,BigDecimal.ROUND_HALF_UP).subtract(new BigDecimal(1)))
                	        		.multiply(new BigDecimal("100"));
                	       /* BigDecimal poor = thisNumber.subtract(lastNumber);
                 	    	BigDecimal complete = poor.divide(lastNumber,2,BigDecimal.ROUND_HALF_UP);
                 	    	BigDecimal changeRatio = complete.multiply(new BigDecimal("100"));*/
                 	    	list.get(0).setChangeRatio(changeRatio.stripTrailingZeros().toPlainString());
            	    	}else{
            	    		//list.get(0).setChangeRatio("100");
            	    		list.get(0).setChangeNumber(thisNumber.toString());
            	    	}
            	    	
            	    	
            	    }
            	    map.put("clxx", list.get(0));
            	    
            	    QueryWrapper<NBasicEdge> ew = new QueryWrapper<>();
            	    ew.eq("house_id", houseId);
            	    if(StringUtils.isEmpty(startTime) && StringUtils.isEmpty(endTime)){
            	    	Calendar calendar1 = Calendar.getInstance();
                 		calendar1.setTime(date);
                 		calendar1.add(Calendar.DAY_OF_MONTH, -6);  // 在当前日基础上-6
                 		//System.out.println(sdf.format(calendar.getTime()));
                 		endTime = sdf.format(date);
                 		startTime = sdf.format(calendar1.getTime());
                 		//System.out.println(startTime+"---------------"+endTime);
                 		map.put("startTime", startTime);
                 		map.put("endTime", endTime);
                 		
            	    }
            	    
            	    ew.apply(!StringUtils.isEmpty(startTime),
                            "date_format (begin_time,'%Y-%m-%d') >= date_format('" + startTime + "','%Y-%m-%d')")
                    .apply(!StringUtils.isEmpty(endTime),
                            "date_format (begin_time,'%Y-%m-%d') <= date_format('" + endTime + "','%Y-%m-%d')");
            	    
            	    ew.orderByDesc("id");
            	    List<NBasicEdge> nBasicEdgeList1 = nBasicEdgeMapper.selectList(ew);
            	    
            		
                    Long startT;
            		try {
            			startT = sdf.parse(startTime).getTime();
            			 Long endT = sdf.parse(endTime).getTime();  
            		     Long oneDay = 1000 * 60 * 60 * 24l;  
            		     Long time = endT;  
            		        while (time >= startT) {  
            		            Date d = new Date(time);  
            		            DateFormat df = new SimpleDateFormat("yyyy-MM-dd");  
            		            String date1=df.format(d);
            		            //System.out.println(date1+"****************");
            		            dayList.add(date1);
            		            /*if(!dayList.contains(date1)){
            		            	
        							wightList.add(0);
            		            }*/
            		            
            		            time -= oneDay; 
            		        }
            		} catch (ParseException e) {
            			// TODO Auto-generated catch block
            			e.printStackTrace();
            		}  
                   
            		
            			for (int j = 0; j < dayList.size(); j++) {
            				if(nBasicEdgeList1.size()>0){
            					for (int i = 0; i < nBasicEdgeList1.size(); i++) {
                					if(dayList.get(j).equals(sdf.format(nBasicEdgeList1.get(i).getBeginTime()))){
        								wightList.add(nBasicEdgeList1.get(i).getLkWeight());
        								break;
        							}
                					if(i == nBasicEdgeList1.size()-1 && wightList.size()<dayList.size()){
                						wightList.add(0);
                					}
                				}
            				}else{
            					wightList.add(0);
            				}
            				
							
						}
					
            		
            	    
            	    map.put("dayList", dayList);
            	    map.put("wightList", wightList);
            	    
        	    }else{
        	    	StorehouseClxx clxx = new StorehouseClxx();
        	    	clxx.setLsNumber("0");
        	    	clxx.setPzName("");
        	    	clxx.setCfName("");
        	    	map.put("clxx", clxx);
        	    	
        	    	
        	    	SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//注意月份是MM
            	    Calendar calendar = Calendar.getInstance();
            	    Date date = new Date();
            		calendar.setTime(date);
            		
            		//System.out.println(sdf.format(calendar.getTime()));
            		for (int i = 0; i < 7; i++) {//默认展示七天的数据
            			wightList.add(0);
            			calendar.add(Calendar.DAY_OF_MONTH, -i);  // 在当前日基础上-1
            			dayList.add(sdf.format(calendar.getTime()));
            			calendar.setTime(date);
					}
            		
        	    	map.put("dayList", dayList);
            	    map.put("wightList", wightList);
            	    map.put("startTime", dayList.get(dayList.size()-1));
             		map.put("endTime", dayList.get(0));
        	    	
        	    }
        	    
        	}
		}
		return map;
	}

	@Override
	public Map<String, Object> getOutInNumber(PageParam pageParam) {
		// TODO Auto-generated method stub
		Map<String, Object> map = new HashMap<>();
		String condition = pageParam.getCondition();
		Integer orgId = 0 ;
        if (!StringUtils.isEmpty(condition)) {
	        
        	JSONObject jsStr = JSON.parseObject(condition);
        	 orgId = jsStr.getIntValue("orgId");
        }
		
		
		QueryWrapper<NBasicEdge> ew = new QueryWrapper<>();//当前天的
		QueryWrapper<NBasicEdge> ew1 = new QueryWrapper<>();//前一天的
		QueryWrapper<NBasicEdge> ew2 = new QueryWrapper<>();//在前一天的
		if (!StringUtils.isEmpty(condition)) {
	        
        	JSONObject jsStr = JSON.parseObject(condition);
        	 orgId = jsStr.getIntValue("orgId");
    		if(!StringUtils.isEmpty(orgId.toString())){
    			ew.eq("fk", orgId);
    			ew1.eq("fk", orgId);
    			ew2.eq("fk", orgId);
    		}
        }
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");//注意月份是MM
		Date date = new Date();
		String time = sdf.format(date);
		
		ew.likeRight("begin_time", time);
		List<NBasicEdge> nBasicEdgeList = nBasicEdgeMapper.selectList(ew);//当天扫描的数据
		
		
		Calendar calendar = Calendar.getInstance();
		calendar.setTime(date);
		calendar.add(Calendar.DAY_OF_MONTH, -1);  // 在当前日基础上-1
		//System.out.println(sdf.format(calendar.getTime()));
		ew1.likeRight("begin_time", sdf.format(calendar.getTime()));
		ew1.orderByDesc("id");
		List<NBasicEdge> lastList = nBasicEdgeMapper.selectList(ew1);//前一天扫描的数据
		
		
		Calendar calendar1 = Calendar.getInstance();
		calendar1.setTime(date);
		calendar1.add(Calendar.DAY_OF_MONTH, -2);  // 在当前日基础上-1
		//System.out.println(sdf.format(calendar1.getTime()));
		ew2.likeRight("begin_time", sdf.format(calendar1.getTime()));
		ew2.orderByDesc("id");
		List<NBasicEdge> lastEstList = nBasicEdgeMapper.selectList(ew2);//前两天扫描的数据
		
		
		BigDecimal outNumber = new BigDecimal(0); //当天出库量
		BigDecimal inNumber = new BigDecimal(0); //当天入库量
		
		BigDecimal lastDayOutNumber = new BigDecimal(0); //前一天出库量
		BigDecimal lastDayInNumber = new BigDecimal(0); //前一天入库量
		
		
		
		if(nBasicEdgeList.size()>0){
			for (int i = 0; i < nBasicEdgeList.size(); i++) {
				BigDecimal number = new BigDecimal(nBasicEdgeList.get(i).getLkWeight());
				//System.out.println(nBasicEdgeList.get(i).getHStatus()+"--------------"+nBasicEdgeList.get(i).getLkWeight());
				if(i == 0){
					BigDecimal lastDayNumber = new BigDecimal(0);

					if(lastList.size()>0){
						 lastDayNumber = new BigDecimal(lastList.get(0).getLkWeight());
					}
					
					if(nBasicEdgeList.get(i).getHStatus().equals("2")){ //2入库 4 出库
						inNumber = inNumber.add(number.subtract(lastDayNumber));
					}
	                if(nBasicEdgeList.get(i).getHStatus().equals("4")){ //2入库 4 出库
	                	outNumber = outNumber.add(number.subtract(lastDayNumber));
					}
				}else{
					BigDecimal lastNumber = new BigDecimal(nBasicEdgeList.get(i-1).getLkWeight());
					if(nBasicEdgeList.get(i).getHStatus().equals("2")){ //2入库 4 出库
						inNumber = inNumber.add(number.subtract(lastNumber));
					}
	                if(nBasicEdgeList.get(i).getHStatus().equals("4")){ //2入库 4 出库
	                	outNumber = outNumber.add(number.subtract(lastNumber));
					}
				}
				
			}
		}
		
		if(lastList.size()>0){
			for (int i = 0; i < lastList.size(); i++) {
				BigDecimal number = new BigDecimal(lastList.get(i).getLkWeight());
				if(lastList.size() == i+1){
					
					BigDecimal lastDayNumber = new BigDecimal(0);
					if(lastEstList.size()>0){
					   lastDayNumber = new BigDecimal(lastEstList.get(0).getLkWeight());
					}
					if(lastList.get(i).getHStatus().equals("2")){ //2入库 4 出库
						lastDayInNumber = lastDayInNumber.add(number.subtract(lastDayNumber));
					}
	                if(lastList.get(i).getHStatus().equals("4")){ //2入库 4 出库
	                	lastDayOutNumber = lastDayOutNumber.add(number.subtract(lastDayNumber));
					}
				}else{
					BigDecimal lastNumber = new BigDecimal(lastList.get(i+1).getLkWeight());
					if(lastList.get(i).getHStatus().equals("2")){ //2入库 4 出库
						lastDayInNumber = lastDayInNumber.add(number.subtract(lastNumber));
					}
	                if(lastList.get(i).getHStatus().equals("4")){ //2入库 4 出库
	                	lastDayOutNumber = lastDayOutNumber.add(number.subtract(lastNumber));
					}
				}
				
			}
		}
		
		/**
		 * 当天出入库量
		 */
		map.put("outNumber", outNumber);
		map.put("inNumber", inNumber);
		
		
		BigDecimal outRatio = new BigDecimal(100);
		BigDecimal inRatio = new BigDecimal(100);
		
		//当期环比增长（下降）率(%)=[(当期价格/上期价格)-1]*100%。
		
		if(lastDayOutNumber.compareTo(BigDecimal.ZERO)!=0){
			
			outRatio = (outNumber.abs().divide(lastDayOutNumber.abs(),2,BigDecimal.ROUND_HALF_UP).subtract(new BigDecimal("1")))
			.multiply(new BigDecimal("100"));
			map.put("outRatio", outRatio);
		   //outRatio = ((outNumber.abs().subtract(lastDayOutNumber.abs())).divide(lastDayOutNumber.abs(),2,BigDecimal.ROUND_HALF_UP)).multiply(new BigDecimal("100"));
		}else{
			map.put("changeOutNumber", outNumber);
		}
		if(lastDayInNumber.compareTo(BigDecimal.ZERO)!=0){
			
			inRatio = (inNumber.divide(lastDayInNumber,2,BigDecimal.ROUND_HALF_UP).subtract(new BigDecimal("1")))
					.multiply(new BigDecimal("100"));
			map.put("inRatio", inRatio);
			 //inRatio = (inNumber.subtract(lastDayInNumber)).divide(lastDayInNumber,2,BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
		}else{
			map.put("changeInNumber", inNumber);
		}
		
		
		
		String sumNumber = null; //总仓容
		String useNumber = null; //已用
		String remainingNumber = null; //剩余可用
		
		
		/**
		 * 总仓容、已用、剩余
		 */
		 useNumber =quantityStatisticsMapper.getUseNumber(orgId);
		 sumNumber = orgInfoMapper.getSumNumber(orgId);
		 BigDecimal sumN = new BigDecimal("0");
		 BigDecimal useN = new BigDecimal("0");
		 if(null != sumNumber){
			  sumN = new BigDecimal(sumNumber);
		 }
		 if(null != useNumber){
			  useN = new BigDecimal(useNumber);

		 }
		 remainingNumber = sumN.subtract(useN).toPlainString();
		 map.put("sumNumber", sumNumber);
		 map.put("useNumber", useNumber);
		 map.put("remainingNumber", remainingNumber);
		return map;
	}

	@Override
	public Map<String, Object> getHouseMessage(PageParam pageParam) {
		// TODO Auto-generated method stub
		Map<String, Object> map = new HashMap<>();
		String condition = pageParam.getCondition();
		if (!StringUtils.isEmpty(condition)) {
	        
        	JSONObject jsStr = JSON.parseObject(condition);
        	Integer orgId = jsStr.getIntValue("orgId");
    		
    		Map<String, Object> parm_map = new HashMap<>();
    		if(!StringUtils.isEmpty(orgId.toString())){
    			parm_map.put("parent_id", orgId);
    		}
    	    parm_map.put("store_type_code", 300003);
    	    List<OrgInfo> list = orgInfoMapper.selectByMap(parm_map);
    	    
    	    //List<StorehouseClxx> list = storehouseClxxMapper.selectByMap(parm_map);
    	    if(list.size()>0){
    	    	for (OrgInfo orgInfo : list) {
    	    		Map<String, Object> mp = new HashMap<>();
    	    	    mp.put("main_id", orgInfo.getId());
    	    	    List<StorehouseClxx> clxxList = storehouseClxxMapper.selectByMap(mp);
    	    	    if(clxxList.size()>0){
    	    	    	BasicEnum basicEnum = basicEnumMapper.selectById(clxxList.get(0).getPz());
        	    	    orgInfo.setPzName(basicEnum.getEnumName());//粮食种类
    	    	    }
    	    	    
            	    QueryWrapper<NBasicEdge> queryWrapper = new QueryWrapper<>();
            	    queryWrapper.eq("house_id", orgInfo.getId());
            	    queryWrapper.orderByDesc("id");
            	    List<NBasicEdge> nBasicEdgeList = nBasicEdgeMapper.selectList(queryWrapper);
            	    if(nBasicEdgeList.size()>0){
            	    	orgInfo.setLsNumber(nBasicEdgeList.get(0).getLkWeight().toString());
            	    }
            	   
				}
    	    }
    	    map.put("list", list);
    	    
		}	
		return map;
	}
	

}
