package com.unissoft.systemManage.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.common.PageParam;
import com.unissoft.systemManage.model.UserInfo;
import com.unissoft.systemManage.mapper.UserInfoMapper;
import com.unissoft.systemManage.service.UserInfoService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-10-19
 */
@Service
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo> implements UserInfoService {
    private UserInfoMapper userMapper;

    @Autowired
    public void setUserMapper(UserInfoMapper userMapper) {
        this.userMapper = userMapper;
    }

    @Override
    public IPage<UserInfo> getUserPage(PageParam pageParam) {
        // 分页条件
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();

        // 查询条件
        String condition = pageParam.getCondition();
        QueryWrapper<UserInfo> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(condition)) {
            // 查询条件转Java对象
            UserInfo userInfo = JSON.parseObject(condition, UserInfo.class);
            // 获取查询条件
            String orgId = userInfo.getOrgId();
            String userRole = userInfo.getUserRole();
            String userName = userInfo.getUserName();
            if (!StringUtils.isEmpty(orgId))
                queryWrapper.eq("org_id", userInfo.getOrgId());
            if (!StringUtils.isEmpty(userRole))
                queryWrapper.eq("user_role", userRole);
            if (!StringUtils.isEmpty(userName))
                queryWrapper.like("user_name", userName);
        }
        queryWrapper.orderByDesc("build_time");
        // 分页对象
        IPage<UserInfo> page = new Page<>(pageIndex, pageSize);
        return userMapper.selectPage(page, queryWrapper);
    }

    @Override
    public List<UserInfo> getUserList(String orgId,String userRole) {
        QueryWrapper<UserInfo> queryWrapper = new QueryWrapper<>();
        if (!StringUtils.isEmpty(orgId)) {
            queryWrapper.eq("org_id", orgId);
        }
    	if(!StringUtils.isEmpty(orgId)){
    		queryWrapper.eq("user_role", userRole);
    	}
    	queryWrapper.eq("user_state", 1);//启用状态
        return userMapper.selectList(queryWrapper);
    }
}
